@@ -3,6 +3,7 @@
    Copyright (C) 2005 Dario Massarin <nekkar@libero.it>
    Copyright (C) 2007-2008 Lukas Appelhans <l.appelhans@gmx.de>
    Copyright (C) 2008 Urs Wolfer <uwolfer @ kde.org>
+   Copyright (C) 2008 Dario Freddi <drf54321@gmail.com>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -37,6 +38,8 @@
 #include <kactioncollection.h>
 #include <kio/renamedialog.h>
 #include <KSystemTrayIcon>
+#include <KSharedConfig>
+#include <KPluginInfo>
 
 #include <QTextStream>
 #include <QDomElement>
@@ -97,7 +100,6 @@ bool KGet::addGroup(const QString& groupName)
         return false;
 
     TransferGroup * group = new TransferGroup(m_transferTreeModel, m_scheduler, groupName);
-    group->handler()->addObserver(new GenericTransferGroupObserver());
     m_transferTreeModel->addGroup(group);
 
     //post notifications
@@ -160,11 +162,7 @@ void KGet::addTransfer(KUrl srcUrl, QString destDir, // krazy:exclude=passbyvalu
     if (destDir.isEmpty())
     {
         if (Settings::useDefaultDirectory())
-#ifdef Q_OS_WIN //krazy:exclude=cpp
-            destDir = Settings::defaultDirectory().remove("file:///");
-#else
-            destDir = Settings::defaultDirectory().remove("file://");
-#endif
+            destDir = KUrl(Settings::defaultDirectory()).path();
 
         QString checkExceptions = getSaveDirectoryFromExceptions(srcUrl);
         if (Settings::enableExceptions() && !checkExceptions.isEmpty())
@@ -211,8 +209,8 @@ void KGet::addTransfer(KUrl::List srcUrls, QString destDir, // krazy:exclude=pas
 {
     KUrl::List urlsToDownload;
 
-    KUrl::List::ConstIterator it = srcUrls.begin();
-    KUrl::List::ConstIterator itEnd = srcUrls.end();
+    KUrl::List::Iterator it = srcUrls.begin();
+    KUrl::List::Iterator itEnd = srcUrls.end();
 
     for(; it!=itEnd ; ++it)
     {
@@ -244,11 +242,7 @@ void KGet::addTransfer(KUrl::List srcUrls, QString destDir, // krazy:exclude=pas
         if (destDir.isEmpty())
         {
             if (Settings::useDefaultDirectory())
-#ifdef Q_OS_WIN //krazy:exclude=cpp
-                destDir = Settings::defaultDirectory().remove("file:///");
-#else
-                destDir = Settings::defaultDirectory().remove("file://");
-#endif
+                destDir = KUrl(Settings::defaultDirectory()).path();
 
             QString checkExceptions = getSaveDirectoryFromExceptions(*it);
             if (Settings::enableExceptions() && !checkExceptions.isEmpty())
@@ -276,9 +270,7 @@ bool KGet::delTransfer(TransferHandler * transfer)
     //the notification to the views that the transfer has been removed, all the
     //pointers to it are invalid.
     transfer->postDeleteEvent();
-// TODO: why does it crash if a download is going to be deleted which is not the last in the list?
-// there are always no problems with the last download.
-//     delete( t );
+    delete t;
     return true;
 }
 
@@ -434,9 +426,11 @@ void KGet::load( QString filename ) // krazy:exclude=passbyvalue
     {
         kWarning(5001) << "Error reading the transfers file";
     }
+
+    addObserver(new GenericModelObserver());
 }
 
-void KGet::save( QString filename ) // krazy:exclude=passbyvalue
+void KGet::save( QString filename, bool plain ) // krazy:exclude=passbyvalue
 {
     if ( !filename.isEmpty()
         && QFile::exists( filename )
@@ -450,31 +444,41 @@ void KGet::save( QString filename ) // krazy:exclude=passbyvalue
     if(filename.isEmpty())
         filename = KStandardDirs::locateLocal("appdata", "transfers.kgt");
 
-    QDomDocument doc(QString("KGetTransfers"));
-    QDomElement root = doc.createElement("Transfers");
-    doc.appendChild(root);
-
-    QList<TransferGroup *>::const_iterator it = m_transferTreeModel->transferGroups().begin();
-    QList<TransferGroup *>::const_iterator itEnd = m_transferTreeModel->transferGroups().end();
-
-    for ( ; it!=itEnd ; ++it )
-    {
-        QDomElement e = doc.createElement("TransferGroup");
-        root.appendChild(e);
-        (*it)->save(e);
-    }
     QFile file(filename);
     if ( !file.open( QIODevice::WriteOnly ) )
     {
         //kWarning(5001)<<"Unable to open output file when saving";
         KMessageBox::error(0,
-                           i18n("Unable to save to: %1", filename),
-                           i18n("Error"));
+                        i18n("Unable to save to: %1", filename),
+                        i18n("Error"));
+
         return;
     }
 
-    QTextStream stream( &file );
-    doc.save( stream, 0 );
+    if (plain) {
+        QTextStream out(&file);
+        foreach(TransferHandler *handler, allTransfers()) {
+            out << handler->source().prettyUrl() << endl;
+        }
+    }
+    else {
+        QDomDocument doc(QString("KGetTransfers"));
+        QDomElement root = doc.createElement("Transfers");
+        doc.appendChild(root);
+
+        QList<TransferGroup *>::const_iterator it = m_transferTreeModel->transferGroups().begin();
+        QList<TransferGroup *>::const_iterator itEnd = m_transferTreeModel->transferGroups().end();
+
+        for ( ; it!=itEnd ; ++it )
+        {
+            QDomElement e = doc.createElement("TransferGroup");
+            root.appendChild(e);
+            (*it)->save(e);
+        }
+
+        QTextStream stream( &file );
+        doc.save( stream, 0 );
+    }
     file.close();
 }
 
@@ -504,25 +508,6 @@ bool KGet::schedulerRunning()
     return (m_scheduler->countRunningJobs() > 0);
 }
 
-void KGet::setPluginsSettingsWidget(KTabWidget * widget)
-{
-    kDebug(5001);
-    QList<TransferFactory *>::iterator it = m_transferFactories.begin();
-    QList<TransferFactory *>::iterator itEnd = m_transferFactories.end();
-
-    QWidget * settingsWidget;
-    for( ; it!=itEnd ; ++it)
-    {
-        KDialog *dialog = static_cast<KDialog*>(widget->parent()->parent());
-        if (!dialog)
-            return;
-
-        settingsWidget = (*it)->createSettingsWidget(dialog);
-        if(settingsWidget)
-            widget->addTab( settingsWidget, (*it)->displayName() );
-    }
-}
-
 QList<TransferHandler*> KGet::allTransfers()
 {
     QList<TransferHandler*> transfers;
@@ -571,8 +556,8 @@ void KGet::settingsChanged()
 {
     kDebug(5001);
 
-    QList<TransferFactory*>::const_iterator it = m_transferFactories.begin();
-    QList<TransferFactory*>::const_iterator itEnd = m_transferFactories.end();
+    QList<TransferFactory*>::const_iterator it = m_transferFactories.constBegin();
+    QList<TransferFactory*>::const_iterator itEnd = m_transferFactories.constEnd();
 
     for( ; it!=itEnd ; ++it )
     {
@@ -600,8 +585,9 @@ QStringList KGet::defaultFolders(const KUrl &filename, const QString &groupname)
     kDebug(5001) << filename << groupname;
 
     QStringList list;
-    if (Settings::enableExceptions() && !KGet::getSaveDirectoryFromExceptions(filename).isEmpty())
-        list.append(KGet::getSaveDirectoryFromExceptions(filename));
+    const QString saveDirectoryFromExceptions = KGet::getSaveDirectoryFromExceptions(filename);
+    if (Settings::enableExceptions() && !saveDirectoryFromExceptions.isEmpty())
+        list.append(saveDirectoryFromExceptions);
 
     TransferGroup * group = KGet::m_transferTreeModel->findGroup(groupname);
 
@@ -620,11 +606,7 @@ QStringList KGet::defaultFolders(const KUrl &filename, const QString &groupname)
 
     for (int i = 0; i < list.size(); i++)
     {
-#ifdef Q_OS_WIN //krazy:exclude=cpp
-        list[i] = list[i].remove("file:///");
-#else
-        list[i] = list[i].remove("file://");
-#endif
+        list[i] = KUrl(list[i]).path();
     }
     return list;
 }
@@ -716,10 +698,10 @@ bool KGet::createTransfer(const KUrl &src, const KUrl &dest, const QString& grou
 
             if(start)
                 newTransfer->handler()->start();
-
+/*
             if (newTransfer->percent() != 100) //Don't add a finished observer if the Transfer has already been finished
                 newTransfer->handler()->addObserver(new GenericTransferObserver());
-
+*/
             return true;
         }
     }
@@ -820,9 +802,9 @@ QString KGet::getSaveDirectoryFromExceptions(const KUrl &filename)
 {
     QString destDir;
 
-    QStringList list = Settings::extensionsFolderList();
-    QStringList::Iterator it = list.begin();
-    QStringList::Iterator end = list.end();
+    const QStringList list = Settings::extensionsFolderList();
+    QStringList::ConstIterator it = list.begin();
+    const QStringList::ConstIterator end = list.end();
     while (it != end) {
         // odd list items are regular expressions for extensions
         QString ext = *it;
@@ -842,10 +824,7 @@ QString KGet::getSaveDirectoryFromExceptions(const KUrl &filename)
         }
     }
 
-#ifdef Q_OS_WIN //krazy:exclude=cpp
-    destDir = destDir.remove("file:///");
-#endif
-    return destDir.remove("file://");
+    return KUrl(destDir).path();
 }
 
 bool KGet::isValidSource(const KUrl &source)
@@ -856,15 +835,15 @@ bool KGet::isValidSource(const KUrl &source)
         KMessageBox::error(0,
                            i18n("Malformed URL:\n%1", source.prettyUrl()),
                            i18n("Error"));
+
         return false;
     }
     // Check if the URL contains the protocol
     if ( source.protocol().isEmpty() ){
-        KMessageBox::error(0,
-                           i18n("Malformed URL:\n%1", source.prettyUrl()),
-// TODO: Replace the string for 4.2 BEFORE string freeze
-//                            i18n("Malformed URL, protocol missing:\n%1", source.prettyUrl()),
+
+        KMessageBox::error(0, i18n("Malformed URL, protocol missing:\n%1", source.prettyUrl()),
                            i18n("Error"));
+
         return false;
     }
     // Check if a transfer with the same url already exists
@@ -882,12 +861,6 @@ bool KGet::isValidSource(const KUrl &source)
             {
                 transfer->stop();
                 TransferHandler * th = transfer->handler();
-                //TODO If the transfer is expanded, it crash.
-                //     transfer should be collapsed before.
-                //     Is correct close it from the KGet class or should it
-                //     be handled in the viewscontainer?
-                //     KGet class should be generic, it shouldn't 
-                //     interfere with the UI
                 KGet::delTransfer(th);
                 return true;
             }
@@ -981,6 +954,7 @@ KUrl KGet::getValidDestUrl(const QString& destDir, const KUrl &srcUrl)
 
 void KGet::loadPlugins()
 {
+    m_transferFactories.clear();
     // Add versioning constraint
     QString
     str  = "[X-KDE-KGet-framework-version] == ";
@@ -1012,25 +986,39 @@ void KGet::loadPlugins()
     //members of this class (why?), such as the m_transferFactories list.
     QList<KGetPlugin *> pluginList;
 
+    KConfigGroup plugins = KConfigGroup(KGlobal::config(), "Plugins");
+
     for( it = services.begin(); it != services.end(); ++it )
     {
+        KPluginInfo info(*it);
+        info.load(plugins);
+
+        if( !info.isPluginEnabled() ) {
+            kDebug(5001) << "TransferFactory plugin (" << (*it)->library()
+                             << ") found, but not enabled";
+            continue;
+        }
+
         KGetPlugin * plugin;
         if( (plugin = createPluginFromService(*it)) != 0 )
         {
+            QString pluginName = info.name();
+            
             pluginList.prepend(plugin);
             kDebug(5001) << "TransferFactory plugin (" << (*it)->library() 
-                      << ") found and added to the list of available plugins" << endl;
+                         << ") found and added to the list of available plugins";
+                
         }
         else
             kDebug(5001) << "Error loading TransferFactory plugin (" 
-                      << (*it)->library() << ")" << endl;
+                      << (*it)->library() << ")";
     }
 
     QList<KGetPlugin *>::iterator it2 = pluginList.begin();
     QList<KGetPlugin *>::iterator it2End = pluginList.end();
 
     for( ; it2!=it2End ; ++it2 )
-        m_transferFactories.append( static_cast<TransferFactory *>(*it2) );
+        m_transferFactories.append( qobject_cast<TransferFactory *>(*it2) );
 
     kDebug(5001) << "Number of factories = " << m_transferFactories.size();
 }
@@ -1042,7 +1030,7 @@ KGetPlugin * KGet::createPluginFromService( const KService::Ptr &service )
 
     if (!factory)
     {
-        KMessageBox::error(0, i18n("<html><p>KPluginFactory could not load the plugin:<br/><i>%1</i></p></html>",
+        KMessageBox::error(0, i18n("<html><p>Plugin loader could not load the plugin:<br/><i>%1</i></p></html>",
                                    service->library()));
         kError(5001) << "KPluginFactory could not load the plugin:" << service->library();
         return 0;
@@ -1059,9 +1047,9 @@ bool KGet::safeDeleteFile( const KUrl& url )
         QFileInfo info( url.path() );
         if ( info.isDir() )
         {
-            KMessageBox::information(0L,i18n("Not deleting\n%1\nas it is a "
-                                     "directory.", url.prettyUrl()),
-                                     i18n("Not Deleted"));
+            KGet::showNotification(m_mainWindow, KNotification::Notification,
+                                   i18n("Not deleting\n%1\nas it is a directory.", url.prettyUrl()),
+                                   "dialog-info");
             return false;
         }
         KIO::NetAccess::del( url, 0L );
@@ -1069,10 +1057,43 @@ bool KGet::safeDeleteFile( const KUrl& url )
     }
 
     else
-        KMessageBox::information( 0L,
-                                  i18n("Not deleting\n%1\nas it is not a local"
-                                          " file.", url.prettyUrl()),
-                                  i18n("Not Deleted") );
+        KGet::showNotification(m_mainWindow, KNotification::Notification,
+                               i18n("Not deleting\n%1\nas it is not a local file.", url.prettyUrl()),
+                               "dialog-info");
     return false;
 }
 
+void KGet::showNotification(QWidget *parent, KNotification::StandardEvent eventId, 
+                            const QString &text, const QString &icon)
+{
+    KNotification::event(eventId, text, KIcon(icon).pixmap(KIconLoader::Small), parent);
+}
+
+GenericModelObserver::GenericModelObserver(QObject *parent) : QObject(parent)
+{
+    m_groupObserver = new GenericTransferGroupObserver();
+}
+
+GenericModelObserver::~GenericModelObserver()
+{
+    delete m_groupObserver;
+}
+
+void GenericModelObserver::addedTransferGroupEvent(TransferGroupHandler * group)
+{
+    Q_UNUSED(group)
+    kDebug() << "OBSERVER :: Adding group " << group;
+    group->addObserver(m_groupObserver);
+
+    // we need to do this to add the genericTransfer to all the transfers under the group
+    m_groupObserver->addTransferGroup(group);
+
+    KGet::save();
+}
+
+void GenericModelObserver::removedTransferGroupEvent(TransferGroupHandler * group)
+{
+    Q_UNUSED(group)
+    group->delObserver(m_groupObserver);
+    KGet::save();
+}
