@@ -70,7 +70,9 @@ void TransferTreeModel::delTransfer(Transfer * transfer)
 {
     TransferGroup * group = transfer->group();
 
-    beginRemoveRows(createIndex(m_transferGroups.indexOf(group), 0, group->handler()), group->size()-1, group->size()-1);
+    // Remove index corresponding to when it's created.
+    int remove_index = group->indexOf(transfer);
+    beginRemoveRows(createIndex(m_transferGroups.indexOf(group), 0, group->handler()), remove_index, remove_index);
 
     group->remove(transfer);
     m_changedTransfers.removeAll(transfer->handler());
@@ -172,14 +174,11 @@ bool TransferTreeModel::isTransferGroup(const QModelIndex & index) const
 
     void * pointer = index.internalPointer();
 
-    foreach(TransferGroup * group, m_transferGroups)
+    Handler *ph = static_cast<Handler*>(pointer);
+    if (dynamic_cast<TransferGroupHandler*>(ph))
     {
-        if(group->handler() == pointer)
-        {
-            return true;
-        }
+        return true;
     }
-
     return false;
 }
 
@@ -431,7 +430,7 @@ QModelIndex TransferTreeModel::index(int row, int column, const QModelIndex & pa
 
     if(isTransferGroup(parent))
     {
-        //The given parent refers to a TransferGroup object
+        //The given parent refers to a TransferGroupHandler object
         void * pointer = parent.internalPointer();
         TransferGroupHandler * group = static_cast<TransferGroupHandler *>(pointer);
 
@@ -553,6 +552,26 @@ bool TransferTreeModel::dropMimeData(const QMimeData * mdata, Qt::DropAction act
     return true;
 }
 
+QString TransferTreeModel::columnName(int column)
+{
+    switch(column) {
+        case TransferTreeModel::Name:
+            return i18n("Name");
+        case TransferTreeModel::Status:
+            return i18n("Status");
+        case TransferTreeModel::Size:
+            return i18n("Size");
+        case TransferTreeModel::Progress:
+            return i18n("Progress");
+        case TransferTreeModel::Speed:
+            return i18n("Speed");
+        case TransferTreeModel::RemainingTime:
+            return i18n("Remaining Time");
+        default:
+            return QString();
+    }
+}
+
 void TransferTreeModel::timerEvent(QTimerEvent *event)
 {
     Q_UNUSED(event);
