@@ -5,8 +5,9 @@
     Copyright (c) 2002-2003 by Martijn Klingens      <klingens@kde.org>
     Copyright (c) 2004      by Richard Smith         <kde@metafoo.co.uk>
     Copyright (c) 2005-2006 by Michaël Larouche     <larouche@kde.org>
+    Copyright (c) 2008      by Roman Jarosz          <kedgedev@centrum.cz>
 
-    Kopete    (c) 2002-2005 by the Kopete developers <kopete-devel@kde.org>
+    Kopete    (c) 2002-2008 by the Kopete developers <kopete-devel@kde.org>
 
     *************************************************************************
     *                                                                       *
@@ -35,6 +36,7 @@
 #include <QtCore/QTextCodec>
 #include <QtCore/QTextStream>
 #include <QtCore/QTimer>
+#include <QtCore/QBuffer>
 #include <QtGui/QClipboard>
 #include <QtGui/QCursor>
 #include <QtGui/QPixmap>
@@ -50,6 +52,8 @@
 #include <dom/html_base.h>
 #include <dom/html_document.h>
 #include <dom/html_inline.h>
+#include <dom/html_form.h>
+#include <dom/dom2_events.h>
 
 
 // KDE includes
@@ -65,7 +69,6 @@
 #include <kstringhandler.h>
 #include <ktemporaryfile.h>
 #include <kio/copyjob.h>
-#include <kstandarddirs.h>
 #include <kstandardaction.h>
 #include <kiconloader.h>
 #include <kcodecs.h>
@@ -87,6 +90,7 @@
 #include "kopeteappearancesettings.h"
 #include "kopetebehaviorsettings.h"
 #include "kopetechatwindowsettings.h"
+#include "kopetetransfermanager.h"
 
 #include "kopetechatwindowstyle.h"
 #include "kopetechatwindowstylemanager.h"
@@ -97,10 +101,11 @@ class ChatMessagePart::Private
 {
 public:
 	Private()
-	 : /*tt(0L),*/ scrollPressed(false), manager(0),
+	 : /*tt(0L),*/ scrollPressed(false), scrollToEndDelayed(false), manager(0),
 	   copyAction(0), saveAction(0), printAction(0),
 	   closeAction(0),copyURLAction(0), currentChatStyle(0),
-	   latestDirection(Kopete::Message::Inbound), latestType(Kopete::Message::TypeNormal)
+	   latestDirection(Kopete::Message::Inbound), latestType(Kopete::Message::TypeNormal),
+	   htmlEventListener(0)
 	{}
 
 	~Private()
@@ -115,6 +120,7 @@ public:
 
 //	ToolTip *tt;
 	bool scrollPressed;
+	bool scrollToEndDelayed;
 	Kopete::ChatSession *manager;
 
 	DOM::HTMLElement activeElement;
@@ -133,6 +139,11 @@ public:
 	// Yep I know it will take memory, but I don't have choice
 	// to enable on-the-fly style changing.
 	QList<Kopete::Message> allMessages;
+
+	// No need to delete, HTMLEventListener is ref counted.
+	QPointer<HTMLEventListener> htmlEventListener;
+
+	QFont chatFont;
 };
 /*
 class ChatMessagePart::ToolTip : public Q3ToolTip
@@ -200,6 +211,7 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 			 KopeteChatWindowSettings::self()->styleName() );
 
 	kDebug(14000) << d->currentChatStyle->getStyleName();
+	connect( this, SIGNAL(completed()), this, SLOT(slotRenderingFinished()) );
 
 	//Security settings, we don't need this stuff
 	setJScriptEnabled( false ) ;
@@ -208,6 +220,9 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 	setMetaRefreshEnabled( false );
 	setOnlyLocalReferences( true );
 
+	// read the font for the chat
+	readChatFont();
+
 	// Write the template to KHTMLPart
 	writeTemplate();
 
@@ -216,6 +231,8 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 
 	connect( Kopete::AppearanceSettings::self(), SIGNAL(messageOverridesChanged()),
 	         this, SLOT( slotAppearanceChanged() ) );
+	connect( Kopete::AppearanceSettings::self(), SIGNAL(appearanceChanged()),
+	         this, SLOT( slotRefreshView() ) );
 	connect( KopeteChatWindowSettings::self(), SIGNAL(chatwindowAppearanceChanged()),
 	         this, SLOT( slotRefreshView() ) );
 	connect( KopeteChatWindowSettings::self(), SIGNAL(styleChanged(const QString &)),
@@ -227,6 +244,9 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 	connect( d->manager, SIGNAL(displayNameChanged()), this, SLOT(slotUpdateHeaderDisplayName()) );
 	connect( d->manager, SIGNAL(photoChanged()), this, SLOT(slotUpdateHeaderPhoto()) );
 
+	connect( d->manager, SIGNAL(messageStateChanged(uint, Kopete::Message::MessageState)),
+	         this, SLOT(messageStateChanged(uint, Kopete::Message::MessageState)) );
+
 	connect ( browserExtension(), SIGNAL( openUrlRequestDelayed( const KUrl &, const KParts::OpenUrlArguments &, const KParts::BrowserArguments & ) ),
 	          this, SLOT( slotOpenURLRequest( const KUrl &, const KParts::OpenUrlArguments &, const KParts::BrowserArguments & ) ) );
 
@@ -235,6 +255,12 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 	connect( view()->verticalScrollBar(), SIGNAL(sliderMoved(int)),
 	         this, SLOT(slotScrollingTo(int)) );
 
+	connect( Kopete::TransferManager::transferManager(), SIGNAL(askIncomingDone(unsigned int)),
+	         this, SLOT(slotFileTransferIncomingDone(unsigned int)) );
+
+	connect( KGlobalSettings::self(), SIGNAL(kdisplayFontChanged()),
+	         this, SLOT( slotRefreshView() ) );
+
 	//initActions
 	d->copyAction = KStandardAction::copy( this, SLOT(copy()), actionCollection() );
 	d->saveAction = KStandardAction::saveAs( this, SLOT(save()), actionCollection() );
@@ -251,6 +277,17 @@ ChatMessagePart::ChatMessagePart( Kopete::ChatSession *mgr, QWidget *parent )
 ChatMessagePart::~ChatMessagePart()
 {
 	kDebug(14000) ;
+
+	// Cancel all pending file transfer requests
+	QList<Kopete::Message>::ConstIterator it, itEnd = d->allMessages.constEnd();
+	for ( it = d->allMessages.constBegin(); it != itEnd; ++it )
+	{
+		if ( (*it).type() == Kopete::Message::TypeFileTransferRequest && !(*it).fileTransferDisabled() )
+		{
+			Kopete::TransferManager::transferManager()->cancelIncomingTransfer( (*it).id() );
+		}
+	}
+
 	//delete d->tt;
 	delete d;
 }
@@ -333,12 +370,27 @@ void ChatMessagePart::slotOpenURLRequest(const KUrl &url, const KParts::OpenUrlA
 	}
 	else
 	{
-		KRun *runner = new KRun( url, 0, false ); // false = non-local files
+		KRun *runner = new KRun( url, 0, 0, false ); // false = non-local files
 		runner->setRunExecutables( false ); //security
 		//KRun autodeletes itself by default when finished.
 	}
 }
 
+void ChatMessagePart::slotFileTransferIncomingDone( unsigned int id )
+{
+	QList<Kopete::Message>::Iterator it = d->allMessages.end();
+	while ( it != d->allMessages.begin() )
+	{
+		--it;
+		if ( (*it).id() == id )
+		{
+			(*it).setFileTransferDisabled( true );
+			disableFileTransferButtons( id );
+			break;
+		}
+	}
+}
+
 void ChatMessagePart::readOverrides()
 {
 	d->bgOverride = Kopete::AppearanceSettings::self()->chatBgOverride();
@@ -373,6 +425,21 @@ void ChatMessagePart::setStyleVariant( const QString &variantPath )
 		variantNode.setInnerText( QString("@import url(\"%1\");").arg( adjustStyleVariantForChatSession( variantPath) ) );
 }
 
+void ChatMessagePart::messageStateChanged( uint messageId, Kopete::Message::MessageState state )
+{
+	QList<Kopete::Message>::Iterator it = d->allMessages.end();
+	while ( it != d->allMessages.begin() )
+	{
+		--it;
+		if ( (*it).id() == messageId )
+		{
+			(*it).setState( state );
+			changeMessageStateElement( messageId, state );
+			break;
+		}
+	}
+}
+
 void ChatMessagePart::slotAppearanceChanged()
 {
 	readOverrides();
@@ -410,7 +477,9 @@ void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 	// Group only if the user want it.
 	if( KopeteChatWindowSettings::self()->groupConsecutiveMessages() )
 	{
-		isConsecutiveMessage = (message.direction() == d->latestDirection && !d->latestContact.isNull() && d->latestContact == message.from() && message.type() == d->latestType);
+		isConsecutiveMessage = (message.direction() == d->latestDirection && !d->latestContact.isNull()
+		                        && d->latestContact == message.from() && message.type() == d->latestType
+		                        && message.type() != Kopete::Message::TypeFileTransferRequest );
 	}
 
 	// Don't test it in the switch to don't break consecutive messages.
@@ -437,6 +506,10 @@ void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 			formattedMessageHtml = d->currentChatStyle->getStatusHtml();
 		}
 	}
+	else if(message.type() == Kopete::Message::TypeFileTransferRequest)
+	{
+		formattedMessageHtml = d->currentChatStyle->getFileTransferIncomingHtml();
+	}
 	else
 	{
 		switch(message.direction())
@@ -497,6 +570,19 @@ void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 		chatNode.appendChild(newMessageNode);
 	}
 
+	if ( message.type() == Kopete::Message::TypeNormal )
+	{
+		if ( message.direction() == Kopete::Message::Outbound )
+			changeMessageStateElement( message.id(), message.state() );
+	}
+	else if ( message.type() == Kopete::Message::TypeFileTransferRequest )
+	{
+		if ( message.fileTransferDisabled() )
+			disableFileTransferButtons( message.id() );
+		else
+			addFileTransferButtonsEventListener( message.id() );
+	}
+
 	// Keep the direction to see on next message
 	// if it's a consecutive message
 	// Keep also the from() contact.
@@ -530,6 +616,9 @@ void ChatMessagePart::appendMessage( Kopete::Message &message, bool restoring )
 
 void ChatMessagePart::slotRefreshView()
 {
+	// refresh the chat font
+	readChatFont();
+
 	DOM::HTMLElement kopeteNode = document().getElementById( QString("KopeteStyle") );
 	if( !kopeteNode.isNull() )
 		kopeteNode.setInnerText( styleHTML() );
@@ -551,17 +640,21 @@ const QString ChatMessagePart::styleHTML() const
 	QString style = QString(
 		"body{background-color:%1;font-family:%2;font-size:%3pt;color:%4}"
 		"td{font-family:%5;font-size:%6pt;color:%7}"
-		"a{color:%8}a.visited{color:%9}"
+		"input{font-family:%8;font-size:%9pt;color:%10}"
+		"a{color:%11}a.visited{color:%12}"
 		"a.KopeteDisplayName{text-decoration:none;color:inherit;}"
 		"a.KopeteDisplayName:hover{text-decoration:underline;color:inherit}"
 		".KopeteLink{cursor:pointer;}.KopeteLink:hover{text-decoration:underline}"
 		".KopeteMessageBody > p:first-child{margin:0;padding:0;display:inline;}" /* some html messages are encapsuled into a <p> */ )
 		.arg( settings->chatBackgroundColor().name() )
-		.arg( settings->chatFont().family() )
-		.arg( settings->chatFont().pointSize() )
+		.arg( d->chatFont.family() )
+		.arg( d->chatFont.pointSize() )
 		.arg( settings->chatTextColor().name() )
-		.arg( settings->chatFont().family() )
-		.arg( settings->chatFont().pointSize() )
+		.arg( d->chatFont.family() )
+		.arg( d->chatFont.pointSize() )
+		.arg( settings->chatTextColor().name() )
+		.arg( d->chatFont.family() )
+		.arg( d->chatFont.pointSize() )
 		.arg( settings->chatTextColor().name() )
 		.arg( settings->chatLinkColor().name() )
 		.arg( settings->chatLinkColor().name() );
@@ -576,6 +669,17 @@ void ChatMessagePart::clear()
 
 	// Reset consecutive messages
 	d->latestContact = 0;
+
+	// Cancel all pending file transfer requests
+	QList<Kopete::Message>::ConstIterator it, itEnd = d->allMessages.constEnd();
+	for ( it = d->allMessages.constBegin(); it != itEnd; ++it )
+	{
+		if ( (*it).type() == Kopete::Message::TypeFileTransferRequest && !(*it).fileTransferDisabled() )
+		{
+			Kopete::TransferManager::transferManager()->cancelIncomingTransfer( (*it).id() );
+		}
+	}
+
 	// Remove all stored messages.
 	d->allMessages.clear();
 }
@@ -583,7 +687,6 @@ void ChatMessagePart::clear()
 Kopete::Contact *ChatMessagePart::contactFromNode( const DOM::Node &n ) const
 {
 	DOM::Node node = n;
-	int i;
 	QList<Kopete::Contact*> m;
 
 	if ( node.isNull() )
@@ -600,7 +703,7 @@ Kopete::Contact *ChatMessagePart::contactFromNode( const DOM::Node &n ) const
 	if ( element.hasAttribute( "contactid" ) )
 	{
 		QString contactId = element.getAttribute( "contactid" ).string();
-		for ( i =0; i != m.size(); i++ )
+		for ( int i =0; i != m.size(); ++i )
 			if ( m.at(i)->contactId() == contactId )
 				return m[i];
 	}
@@ -691,7 +794,7 @@ QString ChatMessagePart::textUnderMouse()
 		cPos = 0,
 		dataLen = data.length();
 
-	QFontMetrics metrics( Kopete::AppearanceSettings::self()->chatFont() );
+	QFontMetrics metrics( d->chatFont );
 	QString buffer;
 	while( cPos < dataLen && nodeLeft < mouseLeft )
 	{
@@ -729,10 +832,20 @@ void ChatMessagePart::slotCopyURL()
 
 void ChatMessagePart::slotScrollView()
 {
-	// NB: view()->contentsHeight() is incorrect before the view has been shown in its window.
-	// Until this happens, the geometry has not been correctly calculated, so this scrollBy call
-	// will usually scroll to the top of the view.
-	view()->scrollBy( 0, view()->contentsHeight() );
+	if ( inProgress() )
+		d->scrollToEndDelayed = true;
+	else
+		view()->scrollBy( 0, view()->contentsHeight() );
+}
+
+void ChatMessagePart::slotRenderingFinished()
+{
+	if ( d->scrollToEndDelayed )
+	{
+		d->scrollToEndDelayed = false;
+		if ( !d->scrollPressed )
+			view()->scrollBy( 0, view()->contentsHeight() );
+	}
 }
 
 void ChatMessagePart::copy(bool justselection /* default false */)
@@ -788,6 +901,7 @@ void ChatMessagePart::khtmlDrawContentsEvent( khtml::DrawContentsEvent * event)
 	KHTMLPart::khtmlDrawContentsEvent(event);
 	//copy(true /*selection only*/); not needed anymore.
 }
+
 void ChatMessagePart::slotCloseView( bool force )
 {
 	d->manager->view()->closeView( force );
@@ -837,18 +951,18 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 
 
 	// Replace sender (contact nick)
-	resultHTML = resultHTML.replace( QLatin1String("%sender%"), nickLink+nick+"</a>" );
+	resultHTML.replace( QLatin1String("%sender%"), nickLink+nick+"</a>" );
 	// Replace time, by default display only time and display seconds(that was true means).
-	if ( Kopete::BehaviorSettings::showDates() )
-		resultHTML = resultHTML.replace( QLatin1String("%time%"), KGlobal::locale()->formatDateTime(message.timestamp(), KLocale::ShortDate, true) );
+	if ( Kopete::BehaviorSettings::showDates() && message.timestamp().date() != QDate::currentDate() )
+		resultHTML.replace( QLatin1String("%time%"), KGlobal::locale()->formatDateTime(message.timestamp(), KLocale::ShortDate, true) );
 	else
-		resultHTML = resultHTML.replace( QLatin1String("%time%"), KGlobal::locale()->formatTime(message.timestamp().time(), true) );
+		resultHTML.replace( QLatin1String("%time%"), KGlobal::locale()->formatTime(message.timestamp().time(), true) );
 	// Replace %screenName% (contact ID)
-	resultHTML = resultHTML.replace( QLatin1String("%senderScreenName%"), nickLink+Qt::escape(contactId)+"</a>" );
+	resultHTML.replace( QLatin1String("%senderScreenName%"), nickLink+Qt::escape(contactId)+"</a>" );
 	// Replace service name (protocol name)
-	resultHTML = resultHTML.replace( QLatin1String("%service%"), Qt::escape(service) );
+	resultHTML.replace( QLatin1String("%service%"), Qt::escape(service) );
 	// Replace protocolIcon (sender statusIcon)
-	resultHTML = resultHTML.replace( QLatin1String("%senderStatusIcon%"), Qt::escape(protocolIcon).replace('"',"&quot;") );
+	resultHTML.replace( QLatin1String("%senderStatusIcon%"), Qt::escape(protocolIcon).replace('"',"&quot;") );
 
 	// Look for %time{X}%
 	QRegExp timeRegExp("%time\\{([^}]*)\\}%");
@@ -856,7 +970,7 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 	while( (pos=timeRegExp.indexIn(resultHTML , pos) ) != -1 )
 	{
 		QString timeKeyword = formatTime( timeRegExp.cap(1), message.timestamp() );
-		resultHTML = resultHTML.replace( pos , timeRegExp.cap(0).length() , timeKeyword );
+		resultHTML.replace( pos , timeRegExp.cap(0).length() , timeKeyword );
 	}
 
 	// Look for %textbackgroundcolor{X}%
@@ -873,7 +987,7 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 	int textPos=0;
 	while( (textPos=textBackgroundRegExp.indexIn(resultHTML, textPos) ) != -1 )
 	{
-		resultHTML = resultHTML.replace( textPos , textBackgroundRegExp.cap(0).length() , bgColor );
+		resultHTML.replace( textPos , textBackgroundRegExp.cap(0).length() , bgColor );
 	}
 
 	// Replace userIconPath
@@ -887,7 +1001,7 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 			else if(message.direction() == Kopete::Message::Outbound)
 				photoPath = d->currentChatStyle->getStyleBaseHref() + QLatin1String("Outgoing/buddy_icon.png");
 		}
-		resultHTML = resultHTML.replace(QLatin1String("%userIconPath%"), photoPath);
+		resultHTML.replace(QLatin1String("%userIconPath%"), photoPath);
 	}
 
 	// Replace messages.
@@ -902,7 +1016,7 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 	}
 
 	// Set message direction("rtl"(Right-To-Left) or "ltr"(Left-to-right))
-	resultHTML = resultHTML.replace( QLatin1String("%messageDirection%"), message.isRightToLeft() ? "rtl" : "ltr" );
+	resultHTML.replace( QLatin1String("%messageDirection%"), message.isRightToLeft() ? "rtl" : "ltr" );
 
 	// These colors are used for coloring nicknames. I tried to use
 	// colors both visible on light and dark background.
@@ -922,7 +1036,7 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 		hash += contactId[f].unicode() * f;
 	const QString colorName = nameColors[ hash % nameColorsLen ];
 	QString lightColorName;	// Do not initialize, QColor::name() is expensive!
-	kDebug(14000) << "Hash " << hash << " has color " << colorName;
+	//kDebug(14000) << "Hash " << hash << " has color " << colorName;
 	QRegExp senderColorRegExp("%senderColor(?:\\{([^}]*)\\})?%");
 	textPos=0;
 	while( (textPos=senderColorRegExp.indexIn(resultHTML, textPos) ) != -1 )
@@ -938,12 +1052,42 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML, const K
 		if ( doLight && lightColorName.isNull() )
 			lightColorName = QColor( colorName ).light( light ).name();
 
-		resultHTML = resultHTML.replace( textPos , senderColorRegExp.cap(0).length(),
+		resultHTML.replace( textPos , senderColorRegExp.cap(0).length(),
 			doLight ? lightColorName : colorName );
 	}
 
+	if ( message.type() == Kopete::Message::TypeFileTransferRequest )
+	{
+		QString fileIcon;
+		if ( !message.filePreview().isNull() )
+		{
+			QByteArray tempArray;
+			QBuffer tempBuffer( &tempArray );
+			tempBuffer.open( QIODevice::WriteOnly );
+			if( message.filePreview().save( &tempBuffer, "PNG" ) )
+				fileIcon = QString( "data:image/png;base64," ) + tempArray.toBase64();
+		}
+
+		if ( fileIcon.isEmpty() )
+		{
+			QString iconName = KMimeType::iconNameForUrl( message.fileName() );
+			fileIcon = KIconLoader::global()->iconPath( iconName, -KIconLoader::SizeMedium );
+		}
+
+		resultHTML.replace( QLatin1String("%fileName%"), Qt::escape( message.fileName() ).replace('"',"&quot;") );
+		resultHTML.replace( QLatin1String("%fileSize%"), KGlobal::locale()->formatByteSize( message.fileSize() ).replace('"',"&quot;") );
+		resultHTML.replace( QLatin1String("%fileIconPath%"), fileIcon );
+
+		resultHTML.replace( QLatin1String("%saveFileHandlerId%"), QString( "ftSV%1" ).arg( message.id() ) );
+		resultHTML.replace( QLatin1String("%saveFileAsHandlerId%"), QString( "ftSA%1" ).arg( message.id() ) );
+		resultHTML.replace( QLatin1String("%cancelRequestHandlerId%"), QString( "ftCC%1" ).arg( message.id() ) );
+	}
+
+	if ( message.type() == Kopete::Message::TypeNormal && message.direction() == Kopete::Message::Outbound )
+		resultHTML.replace( QLatin1String( "%stateElementId%" ), QString( "msST%1" ).arg( message.id() ) );
+
 	// Replace message at the end, maybe someone could put a Adium keyword in his message :P
-	resultHTML = resultHTML.replace( QLatin1String("%message%"), formatMessageBody(message) );
+	resultHTML.replace( QLatin1String("%message%"), formatMessageBody(message) );
 
 	// TODO: %status
 //	resultHTML = addNickLinks( resultHTML );
@@ -970,13 +1114,13 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML )
 			destinationName = remoteContact->nickName();
 
 		// Replace %chatName%, create a internal span to update it by DOM when asked.
-		resultHTML = resultHTML.replace( QLatin1String("%chatName%"), QString("<span id=\"KopeteHeaderChatNameInternal\">%1</span>").arg( formatName(d->manager->displayName(), Qt::RichText) ) );
+		resultHTML.replace( QLatin1String("%chatName%"), QString("<span id=\"KopeteHeaderChatNameInternal\">%1</span>").arg( formatName(d->manager->displayName(), Qt::RichText) ) );
 		// Replace %sourceName%
-		resultHTML = resultHTML.replace( QLatin1String("%sourceName%"), formatName(sourceName, Qt::RichText) );
+		resultHTML.replace( QLatin1String("%sourceName%"), formatName(sourceName, Qt::RichText) );
 		// Replace %destinationName%
-		resultHTML = resultHTML.replace( QLatin1String("%destinationName%"), formatName(destinationName, Qt::RichText) );
+		resultHTML.replace( QLatin1String("%destinationName%"), formatName(destinationName, Qt::RichText) );
 		// For %timeOpened%, display the date and time (also the seconds).
-		resultHTML = resultHTML.replace( QLatin1String("%timeOpened%"), KGlobal::locale()->formatDateTime( QDateTime::currentDateTime(), KLocale::ShortDate, true ) );
+		resultHTML.replace( QLatin1String("%timeOpened%"), KGlobal::locale()->formatDateTime( QDateTime::currentDateTime(), KLocale::ShortDate, true ) );
 
 		// Look for %timeOpened{X}%
 		QRegExp timeRegExp("%timeOpened\\{([^}]*)\\}%");
@@ -984,7 +1128,7 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML )
 		while( (pos=timeRegExp.indexIn(resultHTML, pos) ) != -1 )
 		{
 			QString timeKeyword = formatTime( timeRegExp.cap(1), QDateTime::currentDateTime() );
-			resultHTML = resultHTML.replace( pos , timeRegExp.cap(0).length() , timeKeyword );
+			resultHTML.replace( pos , timeRegExp.cap(0).length() , timeKeyword );
 		}
 		// Get contact image paths
 		QString photoIncoming = photoForContact( remoteContact );
@@ -999,23 +1143,27 @@ QString ChatMessagePart::formatStyleKeywords( const QString &sourceHTML )
 			photoOutgoing = d->currentChatStyle->getStyleBaseHref() + QLatin1String("Outgoing/buddy_icon.png");
 		}
 
-		resultHTML = resultHTML.replace( QLatin1String("%incomingIconPath%"), photoIncoming );
-		resultHTML = resultHTML.replace( QLatin1String("%outgoingIconPath%"), photoOutgoing );
+		resultHTML.replace( QLatin1String("%incomingIconPath%"), photoIncoming );
+		resultHTML.replace( QLatin1String("%outgoingIconPath%"), photoOutgoing );
 	}
 
 	return resultHTML;
 }
 
-QString ChatMessagePart::formatTime(const QString &timeFormat, const QDateTime &dateTime)
+QString ChatMessagePart::formatTime(const QString &_timeFormat, const QDateTime &dateTime)
 {
 	char buffer[256];
-
-	time_t timeT;
-	struct tm *loctime;
+#ifdef Q_WS_WIN
+	QString timeFormat = _timeFormat;
+	// %e is not supported on windows
+	timeFormat = timeFormat.replace("%e", "%d");
+#else
+	const QString timeFormat = _timeFormat;
+#endif
 	// Get current time
-	timeT = dateTime.toTime_t();
+	time_t timeT = dateTime.toTime_t();
 	// Convert it to local time representation.
-	loctime = localtime (&timeT);
+	struct tm* loctime = localtime (&timeT);
 	strftime (buffer, 256, timeFormat.toAscii(), loctime);
 
 	return QString(buffer);
@@ -1170,6 +1318,43 @@ void ChatMessagePart::writeTemplate()
 #endif
 }
 
+void ChatMessagePart::resendMessage( uint messageId )
+{
+	QList<Kopete::Message>::ConstIterator it, itEnd = d->allMessages.constEnd();
+	for ( it = d->allMessages.constBegin(); it != itEnd; ++it )
+	{
+		if ( (*it).id() == messageId )
+		{
+			if ( !( d->manager->protocol()->capabilities() & Kopete::Protocol::CanSendOffline ) )
+			{
+				bool reachableContactFound = false;
+				foreach ( Kopete::Contact* c, (*it).to() )
+				{
+					if ( c->isReachable() )
+					{
+						reachableContactFound = true;
+						break;
+					}
+				}
+
+				// no online contact found and can't send offline? can't send.
+				if ( !reachableContactFound )
+					return;
+			}
+
+			Kopete::Message msg( (*it).from(), (*it).to() );
+			msg.setDirection( Kopete::Message::Outbound );
+			msg.setBody( (*it).body() );
+
+// 			msg.setBackgroundColor( (*it).backgroundColor() );
+			msg.setForegroundColor( (*it).foregroundColor() );
+			msg.setFont( (*it).font() );
+			d->manager->sendMessage( msg );
+			break;
+		}
+	}
+}
+
 QString ChatMessagePart::adjustStyleVariantForChatSession( const QString & styleVariant ) const
 {
 	if ( d->manager->form() == Kopete::ChatSession::Chatroom
@@ -1184,7 +1369,7 @@ QString ChatMessagePart::photoForContact( const Kopete::Contact *contact ) const
 	QString photo;
 	if ( !contact )
 		return photo;
-	
+
 	if( contact->metaContact() == Kopete::ContactList::self()->myself() )
 	{ // all myself contacts have the same metaContact so take photo directly from contact otherwise the photo could be wrong.
 		photo = contact->property(Kopete::Global::Properties::self()->photo().key()).value().toString();
@@ -1193,10 +1378,112 @@ QString ChatMessagePart::photoForContact( const Kopete::Contact *contact ) const
 	{
 		photo = QString( "data:image/png;base64," ) + contact->metaContact()->picture().base64();
 	}
-	
+
 	return photo;
 }
 
+void ChatMessagePart::addFileTransferButtonsEventListener( unsigned int id )
+{
+	QString elementId = QString( "ftSV%1" ).arg( id );
+	registerClickEventListener( document().getElementById( elementId ) );
+
+	elementId = QString( "ftSA%1" ).arg( id );
+	registerClickEventListener( document().getElementById( elementId ) );
+
+	elementId = QString( "ftCC%1" ).arg( id );
+	registerClickEventListener( document().getElementById( elementId ) );
+}
+
+void ChatMessagePart::disableFileTransferButtons( unsigned int id )
+{
+	QString elementId = QString( "ftSV%1" ).arg( id );
+	DOM::HTMLInputElement element = document().getElementById( elementId );
+	if ( !element.isNull() )
+		element.setDisabled( true );
+
+	elementId = QString( "ftSA%1" ).arg( id );
+	element = document().getElementById( elementId );
+	if ( !element.isNull() )
+		element.setDisabled( true );
+
+	elementId = QString( "ftCC%1" ).arg( id );
+	element = document().getElementById( elementId );
+	if ( !element.isNull() )
+		element.setDisabled( true );
+}
+
+void ChatMessagePart::changeMessageStateElement( uint id, Kopete::Message::MessageState state )
+{
+	QString elementId = QString( "msST%1" ).arg( id );
+	DOM::HTMLElement element = document().getElementById( elementId );
+	if ( element.isNull() )
+		return;
+
+	QString statusHTML;
+	switch ( state )
+	{
+	case Kopete::Message::StateUnknown:
+		statusHTML = d->currentChatStyle->getOutgoingStateUnknownHtml();
+		break;
+	case Kopete::Message::StateSending:
+		statusHTML = d->currentChatStyle->getOutgoingStateSendingHtml();
+		break;
+	case Kopete::Message::StateSent:
+		statusHTML = d->currentChatStyle->getOutgoingStateSentHtml();
+		break;
+	case Kopete::Message::StateError:
+		statusHTML = d->currentChatStyle->getOutgoingStateErrorHtml();
+		break;
+	}
+
+	QString resendId = QString( "msRS%1" ).arg( id );
+	statusHTML.replace( QLatin1String( "%resendHandlerId%" ), resendId );
+	element.setInnerHTML( statusHTML );
+
+	registerClickEventListener( document().getElementById( resendId ) );
+}
+
+void ChatMessagePart::registerClickEventListener( DOM::HTMLElement element )
+{
+	if ( element.isNull() )
+		return;
+
+	if ( !d->htmlEventListener )
+	{
+		d->htmlEventListener = new HTMLEventListener();
+		connect( d->htmlEventListener, SIGNAL(resendMessage(uint)), this, SLOT(resendMessage(uint)) );
+	}
+	element.addEventListener( "click", d->htmlEventListener, false );
+}
+
+void ChatMessagePart::readChatFont()
+{
+	Kopete::AppearanceSettings *settings = Kopete::AppearanceSettings::self();
+
+	d->chatFont = KGlobalSettings::generalFont();
+	if ( settings->chatFontSelection() == 1 )
+		d->chatFont = settings->chatFont();
+}
+
+void HTMLEventListener::handleEvent( DOM::Event &event )
+{
+	DOM::HTMLInputElement element = event.currentTarget();
+	if ( !element.isNull() )
+	{
+		QString idType = element.id().string().left(4);
+		unsigned int messageId = element.id().string().mid(4).toUInt();
+
+		if ( idType == QLatin1String( "ftSV" ) )
+			Kopete::TransferManager::transferManager()->saveIncomingTransfer( messageId );
+		else if ( idType == QLatin1String( "ftSA" ) )
+			Kopete::TransferManager::transferManager()->saveIncomingTransfer( messageId );
+		else if ( idType == QLatin1String( "ftCC" ) )
+			Kopete::TransferManager::transferManager()->cancelIncomingTransfer( messageId );
+		else if ( idType == QLatin1String( "msRS" ) )
+			emit resendMessage( messageId );
+	}
+}
+
 #include "chatmessagepart.moc"
 
 // vim: set noet ts=4 sts=4 sw=4:
