@@ -56,7 +56,7 @@ ChatWindowPlugin::ChatWindowPlugin(QObject *parent, const QVariantList &) :
 
 KopeteView* ChatWindowPlugin::createView( Kopete::ChatSession *manager )
 {
-    return (KopeteView*)new ChatView(manager,this);
+    return new ChatView(manager,this);
 }
 
 class KopeteChatViewPrivate
@@ -74,8 +74,6 @@ ChatView::ChatView( Kopete::ChatSession *mgr, ChatWindowPlugin *parent )
 	 : KVBox( 0l ), KopeteView( mgr, parent )
          , d(new KopeteChatViewPrivate)
 {
-	int i;
-
 	d->isActive = false;
 	d->visibleMembers = false;
 	d->sendInProgress = false;
@@ -102,7 +100,7 @@ ChatView::ChatView( Kopete::ChatSession *mgr, ChatWindowPlugin *parent )
 	d->splitter->setChildrenCollapsible( false );
 	QList<int> sizes;
 	sizes << 240 << 40;
-	d->splitter->setSizes( sizes ); 
+	d->splitter->setSizes( sizes );
 
 	// FIXME: is this used these days? it seems totally unnecessary
 	connect( editPart(), SIGNAL( toolbarToggled(bool)), this, SLOT(slotToggleRtfToolbar(bool)) );
@@ -148,13 +146,13 @@ ChatView::ChatView( Kopete::ChatSession *mgr, ChatWindowPlugin *parent )
 
 	// add contacts
 	slotContactAdded( mgr->myself(), true );
-	for ( i = 0; i != mgr->members().size(); i++ )
+	for ( int i = 0; i != mgr->members().size(); ++i )
 	{
 		slotContactAdded( mgr->members()[i], true );
 	}
 
 	setFocusProxy( editPart()->widget() );
-	m_messagePart->widget()->setFocusProxy( editPart()->widget() );
+	m_messagePart->view()->setFocusProxy( editPart()->widget() );
 	editPart()->widget()->setFocus();
 
 	slotChatDisplayNameChanged();
@@ -187,6 +185,15 @@ bool ChatView::canSend()
 	return editPart()->canSend();
 }
 
+bool ChatView::canSendFile()
+{
+	Kopete::ContactPtrList contacts = msgManager()->members();
+	if ( contacts.count() != 1 )
+		return false;
+
+	return contacts.first()->canAcceptFiles();
+}
+
 Kopete::Message ChatView::currentMessage()
 {
 	return editPart()->contents();
@@ -251,6 +258,22 @@ void ChatView::setFont( const QFont &font )
 	editPart()->setFont( font );
 }
 
+void ChatView::resetFontAndColor()
+{
+	Kopete::ContactPtrList contacts = msgManager()->members();
+	if ( contacts.count() != 1 )
+		return;
+
+	Kopete::MetaContact* mc = contacts.first()->metaContact();
+	if ( !mc )
+		return;
+
+	QString contactListGroup = QLatin1String("chatwindow_") + mc->metaContactId();
+	KConfigGroup config = KGlobal::config()->group(contactListGroup);
+	editPart()->resetConfig( config );
+	config.sync();
+}
+
 void ChatView::setFgColor( const QColor &newColor )
 {
     if ( newColor.isValid() )
@@ -371,6 +394,11 @@ bool ChatView::closeView( bool force )
 	return false;
 }
 
+ChatView::KopeteTabState ChatView::tabState() const
+{
+	return m_tabState;
+}
+
 void ChatView::updateChatState( KopeteTabState newState )
 {
 	if ( newState == Undefined )
@@ -521,8 +549,14 @@ void ChatView::slotContactAdded(const Kopete::Contact *contact, bool suppress)
 	if( !suppress && m_manager->members().count() > 1 )
 		sendInternalMessage(  i18n("%1 has joined the chat.", contactName) );
 
+	if ( m_manager->members().count() == 1 )
+		connect( m_manager->members().first(), SIGNAL(canAcceptFilesChanged()), this, SIGNAL(canAcceptFilesChanged()) );
+	else
+		disconnect( m_manager->members().first(), SIGNAL(canAcceptFilesChanged()), this, SIGNAL(canAcceptFilesChanged()) );
+
 	updateChatState();
 	emit updateStatusIcon( this );
+	emit canAcceptFilesChanged();
 }
 
 void ChatView::slotContactRemoved( const Kopete::Contact *contact, const QString &reason, Qt::TextFormat format, bool suppressNotification )
@@ -562,10 +596,13 @@ void ChatView::slotContactRemoved( const Kopete::Contact *contact, const QString
 			else
 				sendInternalMessage( i18n( "%1 has left the chat (%2).", contactName, reason ), format);
 		}
+
+		disconnect( contact, SIGNAL(canAcceptFilesChanged()), this, SIGNAL(canAcceptFilesChanged()) );
 	}
 
 	updateChatState();
 	emit updateStatusIcon( this );
+	emit canAcceptFilesChanged();
 }
 
 QString& ChatView::caption() const
@@ -632,6 +669,17 @@ void ChatView::appendMessage(Kopete::Message &message)
 		unreadMessageFrom.clear();
 }
 
+void ChatView::sendFile()
+{
+	Kopete::ContactPtrList contacts = msgManager()->members();
+	if ( contacts.count() != 1 )
+		return;
+
+	Kopete::Contact* contact = contacts.first();
+	if ( contact->canAcceptFiles() )
+		contact->sendFile();
+}
+
 void ChatView::slotMarkMessageRead()
 {
 	unreadMessageFrom.clear();
@@ -715,15 +763,10 @@ void ChatView::saveOptions()
 void ChatView::saveChatSettings()
 {
 	Kopete::ContactPtrList contacts = msgManager()->members();
-
-	if ( contacts.count() == 0 )
-		return;
-
+	if ( contacts.count() != 1 )
+		return; //can't save with more than one other person in the chat
+	
 	Kopete::MetaContact* mc = contacts.first()->metaContact();
-
-	if ( contacts.count() > 1 )
-		return; //can't save with more than one person in chatview
-
 	if ( !mc )
 		return;
 
@@ -732,13 +775,14 @@ void ChatView::saveChatSettings()
     KConfigGroup config = KGlobal::config()->group(contactListGroup);
 	config.writeEntry( "EnableRichText", editPart()->isRichTextEnabled() );
 	config.writeEntry( "EnableAutoSpellCheck", editPart()->checkSpellingEnabled() );
+	editPart()->writeConfig( config );
 	config.sync();
 }
 
 void ChatView::loadChatSettings()
 {
 	Kopete::ContactPtrList contacts = msgManager()->members();
-	if ( contacts.count() > 1 )
+	if ( contacts.count() != 1 )
 		return; //can't load with more than one other person in the chat
 
 	//read settings for metacontact
@@ -750,6 +794,7 @@ void ChatView::loadChatSettings()
 	emit rtfEnabled( this, editPart()->isRichTextEnabled() );
 	bool enableAutoSpell = config.readEntry( "EnableAutoSpellCheck", Kopete::BehaviorSettings::self()->spellCheck() );
 	emit autoSpellCheckEnabled( this, enableAutoSpell );
+	editPart()->readConfig( config );
 }
 
 void ChatView::readOptions()
@@ -780,6 +825,28 @@ void ChatView::slotRemoteTypingTimeout()
 }
 
 void ChatView::dragEnterEvent ( QDragEnterEvent * event )
+{
+	const bool accept = isDragEventAccepted( event );
+	if ( accept )
+	{
+		event->setAccepted( true );
+		return;
+	}
+	QWidget::dragEnterEvent( event );
+}
+
+void ChatView::dragMoveEvent( QDragMoveEvent * event )
+{
+	const bool accept = isDragEventAccepted( event );
+	if ( accept )
+	{
+		event->setAccepted( true );
+		return;
+	}
+	QWidget::dragMoveEvent( event );
+}
+
+bool ChatView::isDragEventAccepted( QDragMoveEvent * event ) const
 {
 	if( event->provides( "kopete/x-contact" ) )
 	{
@@ -799,7 +866,7 @@ void ChatView::dragEnterEvent ( QDragEnterEvent * event )
 			}
 
 			if(!found && contact != m_manager->myself()->contactId())
-				event->accept();
+				return true;
 		}
 	}
 	else if( event->provides( "kopete/x-metacontact" ) )
@@ -813,22 +880,21 @@ void ChatView::dragEnterEvent ( QDragEnterEvent * event )
 				if( candidate && candidate->account() == m_manager->account() && candidate->isOnline())
 				{
 					if( candidate != m_manager->myself() &&  !m_manager->members().contains( candidate ) )
-						event->accept();
+						return true;
 				}
 			}
 		}
 	}
 	// make sure it doesn't come from the current chat view - then it's an emoticon
-	else if ( event->provides( "text/uri-list" ) && m_manager->members().count() == 1 &&
+	else if ( KUrl::List::canDecode( event->mimeData() ) && m_manager->members().count() == 1 &&
 				 ( event->source() != (QWidget*)m_messagePart->view()->viewport() ) )
 	{
 		Kopete::ContactPtrList members = m_manager->members();
 		Kopete::Contact *contact = members.first();
 		if ( contact && contact->canAcceptFiles() )
-			event->accept();
+			return true;
 	}
-	else
-		QWidget::dragEnterEvent(event);
+	return false;
 }
 
 void ChatView::dropEvent ( QDropEvent * event )
