@@ -583,6 +583,21 @@ void ChatWindowConfig::createPreviewMessages()
 	msgOut2.setPlainBody( i18n( "Ok, an outgoing consecutive message." ) );
 	msgOut2.setDirection( Kopete::Message::Outbound );
 
+	Kopete::Message msgOut3( m_myself, m_jack );
+	msgOut3.setPlainBody( i18n( "Message that is being sent." ) );
+	msgOut3.setDirection( Kopete::Message::Outbound );
+	msgOut3.setState( Kopete::Message::StateSending );
+
+	Kopete::Message msgOut4( m_myself, m_jack );
+	msgOut4.setPlainBody( i18n( "Delivered message." ) );
+	msgOut4.setDirection( Kopete::Message::Outbound );
+	msgOut4.setState( Kopete::Message::StateSent );
+
+	Kopete::Message msgOut5( m_myself, m_jack );
+	msgOut5.setPlainBody( i18n( "Message that can't be delivered." ) );
+	msgOut5.setDirection( Kopete::Message::Outbound );
+	msgOut5.setState( Kopete::Message::StateError );
+
 	Kopete::Message msgCol( m_jack, m_myself );
 	msgCol.setPlainBody( i18n("Here is an incoming colored message.") );
 	msgCol.setDirection( Kopete::Message::Inbound );
@@ -603,6 +618,21 @@ void ChatWindowConfig::createPreviewMessages()
 	msgHigh.setDirection( Kopete::Message::Inbound );
 	msgHigh.setImportance( Kopete::Message::Highlight );
 
+	Kopete::Message msgFTRequest( m_jack, m_myself );
+	msgFTRequest.setPlainBody( i18n( "Hello, this is an incoming file transfer request" ) );
+	msgFTRequest.setDirection( Kopete::Message::Inbound );
+	msgFTRequest.setType( Kopete::Message::TypeFileTransferRequest );
+	msgFTRequest.setFileName( "data.pdf" );
+	msgFTRequest.setFileSize( 10000000 );
+
+	Kopete::Message msgFTRequestDisabled( m_jack, m_myself );
+	msgFTRequestDisabled.setPlainBody( i18n( "Hello, this is a disabled incoming file transfer request" ) );
+	msgFTRequestDisabled.setDirection( Kopete::Message::Inbound );
+	msgFTRequestDisabled.setType( Kopete::Message::TypeFileTransferRequest );
+	msgFTRequestDisabled.setFileName( "data.pdf" );
+	msgFTRequestDisabled.setFileSize( 10000000 );
+	msgFTRequestDisabled.setFileTransferDisabled( true );
+
 	// This is a UTF-8 string btw.
 	Kopete::Message msgRightToLeft( m_myself, m_jack );
 	msgRightToLeft.setPlainBody( i18nc("This special UTF-8 string is to test if the style supports Right-to-Left language display.", "הודעות טקסט") );
@@ -617,10 +647,15 @@ void ChatWindowConfig::createPreviewMessages()
 	m_preview->appendMessage(msgIn2);
 	m_preview->appendMessage(msgOut);
 	m_preview->appendMessage(msgOut2);
+	m_preview->appendMessage(msgOut3);
+	m_preview->appendMessage(msgOut4);
+	m_preview->appendMessage(msgOut5);
 	m_preview->appendMessage(msgCol);
 	m_preview->appendMessage(msgInt);
 	m_preview->appendMessage(msgAct);
 	m_preview->appendMessage(msgHigh);
+	m_preview->appendMessage(msgFTRequest);
+	m_preview->appendMessage(msgFTRequestDisabled);
 	m_preview->appendMessage(msgRightToLeft);
 	m_preview->appendMessage(msgBye);
 }
