@@ -25,6 +25,7 @@
 #include "kircsocket.h"
 
 #include "kopetepasswordedaccount.h"
+#include "kopetemessage.h"
 
 class IRCContact;
 
@@ -134,7 +135,8 @@ public:
 	Kopete::ChatSession *currentCommandSource();
 
 	IRCContact *getContact(const QByteArray &name, Kopete::MetaContact *metac=0);
-	IRCContact *getContact(const KIrc::Entity::Ptr &entity, Kopete::MetaContact *metac=0);
+	IRCContact *getContact(const KIrc::EntityPtr &entity, Kopete::MetaContact *metac=0);
+	QList<Kopete::Contact*> getContacts( const KIrc::EntityList &entities);
 
 	virtual void fillActionMenu( KActionMenu *actionMenu );
 
@@ -168,16 +170,18 @@ private slots:
 
 	void destroyed(IRCContact *contact);
 
-	void receivedEvent(KIrc::Event *event);
+	void receivedEvent(QEvent *event);
 
 	void slotPerformOnConnectCommands();
 
 	void slotShowServerWindow();
 
+	void slotJoinChannel();
+
 private:
 	void clientSetup();
 	void clientConnect();
-
+	void appendMessage(IRCContact* from, QList<Kopete::Contact*> to,const QString& text, Kopete::Message::MessageType type);
 private:
 	Q_DISABLE_COPY(IRCAccount)
 
