@@ -38,14 +38,15 @@
 #include <qstring.h>
 #include <qregexp.h>
 #include <qtimer.h>
+#include <QAbstractSocket>
 
 #include <kcomponentdata.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kmessagebox.h>
+#include <kpassivepopup.h>
 #include <klocale.h>
 #include <kaboutdata.h>
-#include <k3socketbase.h>
 #include <kpassworddialog.h>
 #include <kinputdialog.h>
 #include <kicon.h>
@@ -78,17 +79,12 @@
 
 #include <sys/utsname.h>
 
-#ifdef SUPPORT_JINGLE
-#include "voicecaller.h"
-#include "jinglevoicecaller.h"
-
-// NOTE: Disabled for 0.12, will develop them futher in KDE4
-// #include "jinglesessionmanager.h"
-// #include "jinglesession.h"
-// #include "jinglevoicesession.h"
-#include "jinglevoicesessiondialog.h"
+#ifdef JINGLE_SUPPORT
+#include "jinglecallsmanager.h"
 #endif
 
+#include <sys/utsname.h>
+
 #define KOPETE_CAPS_NODE "http://kopete.kde.org/jabber/caps"
 
 
@@ -104,9 +100,9 @@ JabberAccount::JabberAccount (JabberProtocol * parent, const QString & accountId
 	
 	m_resourcePool = 0L;
 	m_contactPool = 0L;
-#ifdef SUPPORT_JINGLE
-	m_voiceCaller = 0L;
-	//m_jingleSessionManager = 0L; // NOTE: Disabled for 0.12
+
+#ifdef JINGLE_SUPPORT
+	m_jcm = 0L;
 #endif
 	m_bookmarks = new JabberBookmarks(this);
 	m_removing=false;
@@ -146,13 +142,9 @@ void JabberAccount::cleanup ()
 
 	delete m_contactPool;
 	m_contactPool = 0L;
-	
-#ifdef SUPPORT_JINGLE
-	delete m_voiceCaller;
-	m_voiceCaller = 0L;
 
-// 	delete m_jingleSessionManager;
-// 	m_jingleSessionManager = 0L;
+#ifdef JINGLE_SUPPORT
+	delete m_jcm;
 #endif
 }
 
@@ -400,7 +392,7 @@ void JabberAccount::connectWithPassword ( const QString &password )
 	//BEGIN TIMEZONE INFORMATION
 	//
 	// Set timezone information (code from Psi)
-	// Copyright (C) 2001-2003  Justin Karneges
+	// Copyright (C) 2001-2003  Justin Karneges <justin@affinix.com>
 	//
 	time_t x;
 	time(&x);
@@ -608,27 +600,10 @@ void JabberAccount::slotClientError ( JabberClient::ErrorCode errorCode )
 void JabberAccount::slotConnected ()
 {
 	kDebug (JABBER_DEBUG_GLOBAL) << "Connected to Jabber server.";
-	
-#ifdef SUPPORT_JINGLE
-	if(!m_voiceCaller)
-	{
-		kDebug(JABBER_DEBUG_GLOBAL) << "Creating Jingle Voice caller...";
-		m_voiceCaller = new JingleVoiceCaller( this );
-		QObject::connect(m_voiceCaller,SIGNAL(incoming(const Jid&)),this,SLOT(slotIncomingVoiceCall( const Jid& )));
-		m_voiceCaller->initialize();
-	}
-	
-#if 0
-	if(!m_jingleSessionManager)
-	{
-		kDebug(JABBER_DEBUG_GLOBAL) << "Creating Jingle Session Manager...";
-		m_jingleSessionManager = new JingleSessionManager( this );
-		QObject::connect(m_jingleSessionManager, SIGNAL(incomingSession(const QString &, JingleSession *)), this, SLOT(slotIncomingJingleSession(const QString &, JingleSession *)));
-	}
-#endif
 
-	// Set caps extensions
-	m_jabberClient->client()->addExtension("voice-v1", Features(QString("http://www.google.com/xmpp/protocol/voice/v1")));
+#ifdef JINGLE_SUPPORT
+	qDebug() << "Create JingleCallsManager";
+	m_jcm = new JingleCallsManager(this);
 #endif
 
 	kDebug (JABBER_DEBUG_GLOBAL) << "Requesting roster...";
@@ -671,7 +646,7 @@ void JabberAccount::setOnlineStatus( const Kopete::OnlineStatus& status, const K
 	{
 			xmppStatus.setIsAvailable( false );
 			kDebug (JABBER_DEBUG_GLOBAL) << "CROSS YOUR FINGERS! THIS IS GONNA BE WILD";
-            disconnect (Manual, xmppStatus);
+			disconnect (Manual, xmppStatus);
             return;
     }
 
@@ -784,6 +759,11 @@ void JabberAccount::slotCSDisconnected ()
 	 * with the protocol, so clear all resources manually. */
 	resourcePool()->clear();
 
+#ifdef JINGLE_SUPPORT
+	if (m_jcm != 0)
+		delete m_jcm;
+#endif
+
 }
 
 void JabberAccount::handleStreamError (int streamError, int streamCondition, int connectorCode, const QString &server, Kopete::Account::DisconnectReason &errorClass, QString additionalErrMsg)
@@ -854,55 +834,31 @@ void JabberAccount::handleStreamError (int streamError, int streamCondition, int
 		case XMPP::ClientStream::ErrConnection:
 			switch(connectorCode)
 			{
- 				case KNetwork::KSocketBase::LookupFailure:
+ 				case QAbstractSocket::HostNotFoundError:
 					errorClass = Kopete::Account::InvalidHost;
 					errorCondition = i18n("Host not found.");
 					break;
-				case KNetwork::KSocketBase::AddressInUse:
+				case QAbstractSocket::AddressInUseError:
 					errorCondition = i18n("Address is already in use.");
 					break;
-				case KNetwork::KSocketBase::AlreadyCreated:
-					errorCondition = i18n("Cannot recreate the socket.");
-					break;
-				case KNetwork::KSocketBase::AlreadyBound:
-					errorCondition = i18n("Cannot bind the socket again.");
-					break;
-				case KNetwork::KSocketBase::AlreadyConnected:
-					errorCondition = i18n("Socket is already connected.");
-					break;
-				case KNetwork::KSocketBase::NotConnected:
-					errorCondition = i18n("Socket is not connected.");
-					break;
-				case KNetwork::KSocketBase::NotBound:
-					errorCondition = i18n("Socket is not bound.");
-					break;
-				case KNetwork::KSocketBase::NotCreated:
-					errorCondition = i18n("Socket has not been created.");
-					break;
-				case KNetwork::KSocketBase::WouldBlock:
-					errorCondition = i18n("The socket operation would block. You should not see this error: please use \"Report Bug\" from the Help menu.");
-					break;
-				case KNetwork::KSocketBase::ConnectionRefused:
+				case QAbstractSocket::ConnectionRefusedError:
 					errorCondition = i18n("Connection refused.");
 					break;
-				case KNetwork::KSocketBase::ConnectionTimedOut:
-					errorCondition = i18n("Connection timed out.");
-					break;
-				case KNetwork::KSocketBase::InProgress:
+				case QAbstractSocket::UnfinishedSocketOperationError:
 					errorCondition = i18n("Connection attempt already in progress.");
 					break;
-				case KNetwork::KSocketBase::NetFailure:
+				case QAbstractSocket::NetworkError:
 					errorCondition = i18n("Network failure.");
 					break;
-				case KNetwork::KSocketBase::NotSupported:
-					errorCondition = i18n("Operation is not supported.");
-					break;
-				case KNetwork::KSocketBase::Timeout:
+				case QAbstractSocket::SocketTimeoutError:
 					errorCondition = i18n("Socket timed out.");
 					break;
+				case QAbstractSocket::RemoteHostClosedError:
+					errorCondition= i18n("Remote closed connection.");
+					break;
 				default:
 					errorClass = Kopete::Account::ConnectionReset;
-					//errorCondition = i18n("Sorry, something unexpected happened that I do not know more about.");
+					errorCondition = i18n("Unexpected error condition (%1).", connectorCode);
 					break;
 			}
 			if(!errorCondition.isEmpty())
@@ -1034,21 +990,15 @@ void JabberAccount::handleStreamError (int streamError, int streamCondition, int
 			break;
 	}
 
-	/*
-	 * This mustn't be queued as otherwise the reconnection
-	 * API will attempt to reconnect, queueing another
-	 * error until memory is exhausted.
-	 */
 	if(!errorText.isEmpty()) {
 		if (!additionalErrMsg.isEmpty()) {
-			KMessageBox::detailedError (Kopete::UI::Global::mainWidget (),
-					errorText,
-					additionalErrMsg,
-					i18n("Connection problem with Jabber server %1", server));
+			KPassivePopup::message( i18n("Connection problem with Jabber server %1", server),
+			                        QString ("%1\n%2").arg(errorText).arg(additionalErrMsg),
+			                        Kopete::UI::Global::mainWidget() );
 		} else {
-			KMessageBox::error (Kopete::UI::Global::mainWidget (),
-					errorText,
-					i18n("Connection problem with Jabber server %1", server));
+			KPassivePopup::message( i18n("Connection problem with Jabber server %1", server),
+			                        errorText,
+			                        Kopete::UI::Global::mainWidget() );
 		}
 	}
 
@@ -1071,10 +1021,12 @@ void JabberAccount::slotCSError ( int error )
 		kDebug ( JABBER_DEBUG_GLOBAL ) << "Disconnecting.";
 
 		// display message to user
-		if(!m_removing) //when removing the account, connection errors are normal.
+		// when removing or disconnecting, connection errors are normal
+		if(!m_removing && (isConnected() || isConnecting()))
 			handleStreamError (error, client()->clientStream()->errorCondition (), client()->clientConnector()->errorCode (), server (), errorClass, client()->clientStream()->errorText());
 
-		disconnect ( errorClass );
+		if (isConnected() || isConnecting())
+			disconnect ( errorClass );
 		
 		/*	slotCSDisconnected  will not be called*/
 		resourcePool()->clear();
@@ -1396,7 +1348,7 @@ void JabberAccount::slotContactUpdated (const XMPP::RosterItem & item)
 		if(metaContact->isTemporary())
 			return;
 		kDebug (JABBER_DEBUG_GLOBAL) << c->contactId() << 
-				" is on the contact list while it shouldn't.  we are removing it.  - " << c << endl;
+				" is on the contact list while it should not.  we are removing it.  - " << c << endl;
 		delete c;
 		if(metaContact->contacts().isEmpty())
 			Kopete::ContactList::self()->removeMetaContact( metaContact );
@@ -1436,7 +1388,7 @@ void JabberAccount::slotReceivedMessage (const XMPP::Message & message)
 		 */
 		if ( !contactFrom )
 		{
-			kDebug ( JABBER_DEBUG_GLOBAL ) << "WARNING: Received a groupchat message but couldn't find room contact. Ignoring message.";
+			kDebug ( JABBER_DEBUG_GLOBAL ) << "WARNING: Received a groupchat message but could not find room contact. Ignoring message.";
 			return;
 		}
 	}
@@ -1562,7 +1514,7 @@ void JabberAccount::slotGroupChatPresence (const XMPP::Jid & jid, const XMPP::St
 
 	if ( !groupContact )
 	{
-		kDebug ( JABBER_DEBUG_GLOBAL ) << "WARNING: Groupchat presence signalled, but we don't have a room contact?";
+		kDebug ( JABBER_DEBUG_GLOBAL ) << "WARNING: Groupchat presence signalled, but we do not have a room contact?";
 		return;
 	}
 
@@ -1665,21 +1617,21 @@ void JabberAccount::slotEditVCard ()
 	static_cast<JabberContact *>( myself() )->slotUserInfo ();
 }
 
-const QString JabberAccount::resource () const
+QString JabberAccount::resource () const
 {
 
 	return configGroup()->readEntry ( "Resource", "Kopete" );
 
 }
 
-const QString JabberAccount::server () const
+QString JabberAccount::server () const
 {
 
 	return configGroup()->readEntry ( "Server" );
 
 }
 
-const int JabberAccount::port () const
+int JabberAccount::port () const
 {
 
 	return configGroup()->readEntry ( "Port", 5222 );
@@ -1694,38 +1646,6 @@ void JabberAccount::slotGetServices ()
 	dialog->raise ();
 }
 
-void JabberAccount::slotIncomingVoiceCall( const Jid &jid )
-{
-	kDebug(JABBER_DEBUG_GLOBAL) ;
-#ifdef SUPPORT_JINGLE
-	if(voiceCaller())
-	{
-		kDebug(JABBER_DEBUG_GLOBAL) << "Showing voice dialog.";
-		JingleVoiceSessionDialog *voiceDialog = new JingleVoiceSessionDialog( jid, voiceCaller() );
-		voiceDialog->show();
-	}
-#else
-	Q_UNUSED(jid);
-#endif
-}
-
-// void JabberAccount::slotIncomingJingleSession( const QString &sessionType, JingleSession *session )
-// {
-// #ifdef SUPPORT_JINGLE
-// 	if(sessionType == "http://www.google.com/session/phone")
-// 	{
-// 		QString from = ((XMPP::Jid)session->peers().first()).full();
-// 		//KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Information, QString("Received a voice session invitation from %1.").arg(from) );
-// 		JingleVoiceSessionDialog *voiceDialog = new JingleVoiceSessionDialog( static_cast<JingleVoiceSession*>(session) );
-// 		voiceDialog->show();
-// 	}
-// #else
-// 	Q_UNUSED( sessionType );
-// 	Q_UNUSED( session );
-// #endif
-// }
-
-
 void JabberAccount::addTransport( JabberTransport * tr, const QString &jid )
 {
 	m_transports.insert(jid,tr);
@@ -1815,7 +1735,6 @@ void JabberMoodAction::triggered()
 	emit triggered(mType);
 }*/
 
-
 #include "jabberaccount.moc"
 
 // vim: set noet ts=4 sts=4 sw=4:
