@@ -137,6 +137,7 @@ private slots:
 	void slotStartChat();
 	void slotOpenInbox();
 	void slotChangePublicName();
+	void slotOpenStatus();
 
 //#if !defined NDEBUG //(Stupid moc which don't see when he don't need to slot this slot)
 	/**
@@ -205,6 +206,7 @@ private:
 	KAction *m_openInboxAction;
 	KAction *m_startChatAction;
 	KAction *m_changeDNAction;
+	KAction *m_openStatusAction;
 
 	// status which will be using for connecting
 	Kopete::OnlineStatus m_connectstatus;
