@@ -33,7 +33,6 @@ namespace Xtraz { class Status; }
 class ICQProtocol;
 class ICQAccount;
 class ICQUserInfoWidget;
-class ICQContact;
 
 class ICQMyselfContact : public OscarMyselfContact
 {
@@ -111,7 +110,6 @@ private:
 	bool mHideIP;
 	Kopete::StatusMessage mInitialStatusMessage;
 	ICQUserInfoWidget* mInfoWidget;
-	ICQContact* mInfoContact;
 
 	KAction* mEditInfoAction;
 	KToggleAction* mActionInvisible;
