@@ -2,12 +2,12 @@
 	client.cpp - Kopete Oscar Protocol
 
 	Copyright (c) 2004-2005 Matt Rogers <mattr@kde.org>
-    Copyright (c) 2007 Roman Jarosz <kedgedev@centrum.cz>
+    Copyright (c) 2008 Roman Jarosz <kedgedev@centrum.cz>
 
 	Based on code Copyright (c) 2004 SuSE Linux AG <http://www.suse.com>
-	Based on Iris, Copyright (C) 2003  Justin Karneges
+	Based on Iris, Copyright (C) 2003  Justin Karneges <justin@affinix.com>
 
-	Kopete (c) 2002-2007 by the Kopete developers <kopete-devel@kde.org>
+	Kopete (c) 2002-2008 by the Kopete developers <kopete-devel@kde.org>
 
 	*************************************************************************
 	*                                                                       *
@@ -43,6 +43,7 @@
 #include "logintask.h"
 #include "connection.h"
 #include "messagereceivertask.h"
+#include "messageacktask.h"
 #include "onlinenotifiertask.h"
 #include "oscarclientstream.h"
 #include "oscarsettings.h"
@@ -72,6 +73,7 @@
 #include "closeconnectiontask.h"
 #include "icqtlvinforequesttask.h"
 #include "icqtlvinfoupdatetask.h"
+#include "filetransferhandler.h"
 
 
 namespace
@@ -122,6 +124,7 @@ public:
 	OnlineNotifierTask* onlineNotifier;
 	OwnUserInfoTask* ownStatusTask;
 	MessageReceiverTask* messageReceiverTask;
+	MessageAckTask* messageAckTask;
 	SSIAuthTask* ssiAuthTask;
 	ICQUserInfoRequestTask* icqInfoTask;
 	ICQTlvInfoRequestTask* icqTlvInfoTask;
@@ -143,7 +146,8 @@ public:
 		Oscar::DWORD status;
 		QString message;     // for away-,DND-message etc., and for Xtraz status
 		int xtraz;           // Xtraz status
-		QString description; // Xtraz description
+		int mood;            // Mood
+		QString title;       // Xtraz/Mood title
 		bool sent;
 	} status;
 
@@ -175,6 +179,7 @@ Client::Client( QObject* parent )
 	d->offlineMessagesRequested = false;
 	d->status.status = 0x0; // default to online
 	d->status.xtraz = -1; // default to no Xtraz
+	d->status.mood = -1;
 	d->status.sent = false;
 	d->ssiManager = new ContactManager( this );
 	d->settings = new Oscar::Settings();
@@ -182,6 +187,7 @@ Client::Client( QObject* parent )
 	d->onlineNotifier = 0L;
 	d->ownStatusTask = 0L;
 	d->messageReceiverTask = 0L;
+	d->messageAckTask = 0L;
 	d->ssiAuthTask = 0L;
 	d->icqInfoTask = 0L;
 	d->icqTlvInfoTask = 0L;
@@ -248,7 +254,13 @@ void Client::close()
 {
 	QList<Connection*> cList = d->connections.connections();
 	for ( int i = 0; i < cList.size(); i++ )
-		(new CloseConnectionTask( cList.at(i)->rootTask() ))->go( Task::AutoDelete );
+	{
+		Connection* c = cList.at(i);
+		(new CloseConnectionTask( c->rootTask() ))->go( Task::AutoDelete );
+		
+		foreach ( Oscar::MessageInfo info, c->messageInfoList() )
+			emit messageError( info.contact, info.id );
+	}
 
 	d->active = false;
 	d->awayMsgRequestTimer->stop();
@@ -262,9 +274,10 @@ void Client::close()
 	{
 		d->status.status = 0x0;
 		d->status.xtraz = -1;
+		d->status.mood = -1;
 		d->status.sent = false;
 		d->status.message.clear();
-		d->status.description.clear();
+		d->status.title.clear();
 	}
 
 	d->exchanges.clear();
@@ -275,17 +288,19 @@ void Client::close()
 	d->offlineMessagesRequested = false;
 }
 
-void Client::setStatus( Oscar::DWORD status, const QString &message, int xtraz, const QString &description )
+void Client::setStatus( Oscar::DWORD status, const QString &message, int xtraz, const QString &title, int mood )
 {
 	kDebug(OSCAR_RAW_DEBUG) << "Setting status message to "<< message;
 
 	// remember the values to reply with, when requested
 	bool xtrazChanged = (xtraz > -1 || d->status.xtraz != xtraz);
-	bool statusInfoChanged = ( !d->status.sent || message != d->status.message || description != d->status.description );
+	bool moodChanged = (mood > -1 || d->status.mood != mood);
+	bool statusInfoChanged = ( !d->status.sent || message != d->status.message || title != d->status.title );
 	d->status.status = status;
 	d->status.message = message;
 	d->status.xtraz = xtraz;
-	d->status.description = description;
+	d->status.mood = mood;
+	d->status.title = title;
 	d->status.sent = false;
 
 	if ( d->active )
@@ -315,7 +330,23 @@ void Client::setStatus( Oscar::DWORD status, const QString &message, int xtraz,
 		if ( !c )
 			return;
 
+		if ( d->isIcq && statusInfoChanged )
+		{
+			ICQFullInfo info( false );
+			info.statusDescription.set( title.toUtf8() );
+			
+			ICQTlvInfoUpdateTask* infoUpdateTask = new ICQTlvInfoUpdateTask( c->rootTask() );
+			infoUpdateTask->setInfo( info );
+			infoUpdateTask->go( Task::AutoDelete );
+		}
+
 		SendDCInfoTask* sdcit = new SendDCInfoTask( c->rootTask(), status );
+		if ( d->isIcq && moodChanged )
+			sdcit->setIcqMood( mood );
+
+		if ( d->isIcq && statusInfoChanged )
+			sdcit->setIcqMessage( title );
+
 		sdcit->go( Task::AutoDelete ); //autodelete
 
 		QString msg;
@@ -342,15 +373,6 @@ void Client::setStatus( Oscar::DWORD status, const QString &message, int xtraz,
 
 		pt->go( Task::AutoDelete );
 
-		if ( d->isIcq && statusInfoChanged )
-		{
-			ICQFullInfo info( false );
-			info.statusDescription.set( description.toUtf8() );
-
-			ICQTlvInfoUpdateTask* infoUpdateTask = new ICQTlvInfoUpdateTask( c->rootTask() );
-			infoUpdateTask->setInfo( info );
-			infoUpdateTask->go( Task::AutoDelete );
-		}
 		d->status.sent = true;
 	}
 }
@@ -457,7 +479,7 @@ void Client::serviceSetupFinished()
 {
 	d->active = true;
 
-	setStatus( d->status.status, d->status.message, d->status.xtraz, d->status.description );
+	setStatus( d->status.status, d->status.message, d->status.xtraz, d->status.title, d->status.mood );
 	d->ownStatusTask->go();
 
 	emit haveContactList();
@@ -475,7 +497,7 @@ void Client::receivedIcqInfo( const QString& contact, unsigned int type )
 		emit receivedIcqLongInfo( contact );
 }
 
-void Client::receivedInfo( quint16 sequence )
+void Client::receivedInfo( Oscar::DWORD sequence )
 {
 	UserDetails details = d->userInfoTask->getInfoFor( sequence );
 	emit receivedUserInfo( details.userId(), details );
@@ -538,9 +560,14 @@ int Client::statusXtraz() const
 	return d->status.xtraz;
 }
 
-QString Client::statusDescription() const
+int Client::statusMood() const
 {
-	return d->status.description;
+	return d->status.mood;
+}
+
+QString Client::statusTitle() const
+{
+	return d->status.title;
 }
 
 QString Client::statusMessage() const
@@ -630,7 +657,7 @@ void Client::receivedMessage( const Oscar::Message& msg )
 						{
 							XtrazNotify xNotifyResponse;
 							xNotifyResponse.setSenderUni( userId() );
-							response.setPlugin( xNotifyResponse.statusResponse( statusXtraz(), statusDescription(), statusMessage() ) );
+							response.setPlugin( xNotifyResponse.statusResponse( statusXtraz(), statusTitle(), statusMessage() ) );
 							emit userReadsStatusMessage( msg.sender() );
 						}
 					}
@@ -787,6 +814,7 @@ void Client::initializeStaticTasks()
 	d->onlineNotifier = new OnlineNotifierTask( c->rootTask() );
 	d->ownStatusTask = new OwnUserInfoTask( c->rootTask() );
 	d->messageReceiverTask = new MessageReceiverTask( c->rootTask() );
+	d->messageAckTask = new MessageAckTask( c->rootTask() );
 	d->ssiAuthTask = new SSIAuthTask( c->rootTask() );
 	d->icqInfoTask = new ICQUserInfoRequestTask( c->rootTask() );
 	d->icqTlvInfoTask = new ICQTlvInfoRequestTask( c->rootTask() );
@@ -808,6 +836,11 @@ void Client::initializeStaticTasks()
 	connect( d->messageReceiverTask, SIGNAL( fileMessage( int, const QString, const QByteArray, Buffer ) ),
 	         this, SLOT( gotFileMessage( int, const QString, const QByteArray, Buffer ) ) );
 
+	connect( d->messageAckTask, SIGNAL(messageAck(const QString&, uint)),
+	         this, SIGNAL(messageAck(const QString&, uint)) );
+	connect( d->errorTask, SIGNAL(messageError(const QString&, uint)),
+	         this, SIGNAL(messageError(const QString&, uint)) );
+	
 	connect( d->ssiAuthTask, SIGNAL( authRequested( const QString&, const QString& ) ),
 	         this, SIGNAL( authRequestReceived( const QString&, const QString& ) ) );
 	connect( d->ssiAuthTask, SIGNAL( authReplied( const QString&, const QString&, bool ) ),
@@ -1638,6 +1671,9 @@ void Client::determineDisconnection( int code, const QString& string )
 		emit socketError( code, string );
 	}
 
+	foreach ( Oscar::MessageInfo info, c->messageInfoList() )
+		emit messageError( info.contact, info.id );
+
     //connection is deleted. deleteLater() is used
     d->connections.remove( c );
     c = 0;
@@ -1704,6 +1740,7 @@ void Client::deleteStaticTasks()
 	delete d->onlineNotifier;
 	delete d->ownStatusTask;
 	delete d->messageReceiverTask;
+	delete d->messageAckTask;
 	delete d->ssiAuthTask;
 	delete d->icqInfoTask;
 	delete d->icqTlvInfoTask;
@@ -1715,6 +1752,7 @@ void Client::deleteStaticTasks()
 	d->onlineNotifier = 0;
 	d->ownStatusTask = 0;
 	d->messageReceiverTask = 0;
+	d->messageAckTask = 0;
 	d->ssiAuthTask = 0;
 	d->icqInfoTask = 0;
 	d->icqTlvInfoTask = 0;
@@ -1729,16 +1767,17 @@ bool Client::hasIconConnection( ) const
 	return c;
 }
 
-void Client::sendFiles( const QString& contact, const QStringList& files, Kopete::Transfer *t )
+FileTransferHandler* Client::createFileTransfer( const QString& contact, const QStringList& files )
 {
 	Connection* c = d->connections.connectionForFamily( 0x0004 );
 	if ( !c )
-		return;
+		return 0;
 
-	FileTransferTask *ft = new FileTransferTask( c->rootTask(), contact, ourInfo().userId(), files, t );
+	FileTransferTask *ft = new FileTransferTask( c->rootTask(), contact, ourInfo().userId(), files );
 	connect( ft, SIGNAL( sendMessage( const Oscar::Message& ) ),
 	         this, SLOT( fileMessage( const Oscar::Message& ) ) );
-	ft->go( Task::AutoDelete );
+
+	return new FileTransferHandler(ft);
 }
 
 void Client::gotFileMessage( int type, const QString from, const QByteArray cookie, Buffer buf)
@@ -1760,14 +1799,11 @@ void Client::gotFileMessage( int type, const QString from, const QByteArray cook
 	{
 		kDebug(14151) << "new request :)";
 		FileTransferTask *ft = new FileTransferTask( c->rootTask(), from, ourInfo().userId(), cookie, buf );
-		connect( ft, SIGNAL( getTransferManager( Kopete::TransferManager ** ) ),
-				SIGNAL( getTransferManager( Kopete::TransferManager ** ) ) );
-		connect( ft, SIGNAL( askIncoming( QString, QString, Oscar::DWORD, QString, QString ) ),
-				SIGNAL( askIncoming( QString, QString, Oscar::DWORD, QString, QString ) ) );
 		connect( ft, SIGNAL( sendMessage( const Oscar::Message& ) ),
 				this, SLOT( fileMessage( const Oscar::Message& ) ) );
 		ft->go( Task::AutoDelete );
-		return;
+		
+		emit incomingFileTransfer( new FileTransferHandler(ft) );
 	}
 
 	kDebug(14151) << "nobody wants it :(";
