@@ -5,7 +5,7 @@
 	Copyright (c) 2004-2005 by Matt Rogers <mattr@kde.org>
 
 	Based on code Copyright (c) 2004 SuSE Linux AG <http://www.suse.com>
-	Based on Iris, Copyright (C) 2003  Justin Karneges
+	Based on Iris, Copyright (C) 2003  Justin Karneges <justin@affinix.com>
 
 	Kopete (c) 2002-2005 by the Kopete developers <kopete-devel@kde.org>
 
@@ -35,7 +35,8 @@
 class ConnectionPrivate
 {
 public:
-	Oscar::DWORD snacSequence;
+	// The Oscar::WORD isn't typo see snacSequence function
+	Oscar::WORD snacSequence;
 	Oscar::WORD flapSequence;
 
 	QList<int> familyList;
@@ -45,6 +46,7 @@ public:
 	Client* client;
 
 	Task* root;
+	QHash<Oscar::DWORD, Oscar::MessageInfo> snacSequenceToMessageInfo;
 };
 
 
@@ -125,7 +127,7 @@ Oscar::Settings* Connection::settings() const
 	return d->client->clientSettings();
 }
 
-quint16 Connection::flapSequence()
+Oscar::WORD Connection::flapSequence()
 {
 	d->flapSequence++;
 	if ( d->flapSequence >= 0x8000 ) //the max flap sequence is 0x8000 ( HEX )
@@ -134,9 +136,13 @@ quint16 Connection::flapSequence()
 	return d->flapSequence;
 }
 
-quint32 Connection::snacSequence()
+Oscar::DWORD Connection::snacSequence()
 {
 	d->snacSequence++;
+
+	if ( d->snacSequence == 0x0000 )
+		d->snacSequence++;
+
 	return d->snacSequence;
 }
 
@@ -185,6 +191,21 @@ QHostAddress Connection::localAddress() const
 	return d->clientStream->localAddress();
 }
 
+void Connection::addMessageInfo( Oscar::DWORD snacSequence, const Oscar::MessageInfo& messageInfo )
+{
+	d->snacSequenceToMessageInfo.insert( snacSequence, messageInfo );
+}
+
+Oscar::MessageInfo Connection::takeMessageInfo( Oscar::DWORD snacSequence )
+{
+	return d->snacSequenceToMessageInfo.take( snacSequence );
+}
+
+QList<Oscar::MessageInfo> Connection::messageInfoList() const
+{
+	return d->snacSequenceToMessageInfo.values();
+}
+
 RateClassManager* Connection::rateManager() const
 {
 	return d->rateClassManager;
@@ -231,6 +252,7 @@ void Connection::reset()
 	//clear the family list
 	d->familyList.clear();
 	d->rateClassManager->reset();
+	d->snacSequenceToMessageInfo.clear();
 }
 
 void Connection::streamReadyRead()
