@@ -1,11 +1,11 @@
 /*
     Kopete Yahoo Protocol
     
-    Copyright (c) 2005-2006 André Duffeck <andre.duffeck@kdemail.net>
+    Copyright (c) 2005-2006 André Duffeck <duffeck@kde.org>
     Copyright (c) 2004 Duncan Mac-Vicar P. <duncan@kde.org>
     Copyright (c) 2004 Matt Rogers <matt.rogers@kdemail.net>
     Copyright (c) 2004 SuSE Linux AG <http://www.suse.com>
-    Copyright (C) 2003  Justin Karneges
+    Copyright (C) 2003  Justin Karneges <justin@affinix.com>
     
     Kopete (c) 2002-2006 by the Kopete developers <kopete-devel@kde.org>
  
@@ -19,11 +19,11 @@
     *************************************************************************
 */
 
-#include <qtimer.h>
+#include <QTimer>
+#include <QPixmap>
 
 #include <kdebug.h>
-#include <kurl.h>
-#include <ksocketbase.h>
+#include <k3socketbase.h>
 
 #include "yahooclientstream.h"
 #include "yahooconnector.h"
@@ -52,6 +52,7 @@
 #include "sendfiletask.h"
 #include "filetransfernotifiertask.h"
 #include "receivefiletask.h"
+#include "yahoochattask.h"
 #include "client.h"
 #include "yahootypes.h"
 #include "yahoobuddyiconloader.h"
@@ -86,6 +87,7 @@ public:
 	ConferenceTask *conferenceTask;
 	YABTask *yabTask;
 	FileTransferNotifierTask *fileTransferTask;
+	YahooChatTask *yahooChatTask;
 
 	// Connection data
 	uint sessionID;
@@ -95,11 +97,13 @@ public:
 	Yahoo::Status status;
 	Yahoo::Status statusOnConnect;
 	QString statusMessageOnConnect;
-	int pictureFlag;
+	Yahoo::PictureStatus pictureFlag;
+	int pictureChecksum;
 };
 
-Client::Client(QObject *par) :QObject(par, "yahooclient" )
+Client::Client(QObject *par) :QObject(par)
 {
+	setObjectName( QLatin1String("yahooclient") );
 	d = new ClientPrivate;
 /*	d->tzoffset = 0;*/
 	d->active = false;
@@ -112,7 +116,7 @@ Client::Client(QObject *par) :QObject(par, "yahooclient" )
 	d->iconLoader = 0L;
 	d->loginTask = new LoginTask( d->root );
 	d->listTask = new ListTask( d->root );
-	d->pictureFlag = 0;
+	d->pictureFlag = Yahoo::NoPicture;
 	m_connector = 0L;
 
 	m_pingTimer = new QTimer( this );
@@ -138,7 +142,7 @@ Client::~Client()
 
 void Client::connect( const QString &host, const uint port, const QString &userId, const QString &pass )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << endl;
+	kDebug(YAHOO_RAW_DEBUG) ;
 	d->host = host;
 	d->port = port;
 	d->user = userId;
@@ -162,9 +166,9 @@ void Client::cancelConnect()
 
 void Client::cs_connected()
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << endl;
+	kDebug(YAHOO_RAW_DEBUG) ;
 	emit connected();
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << " starting login task ... "<<  endl;
+	kDebug(YAHOO_RAW_DEBUG) << " starting login task ... ";
 
 	d->loginTask->setStateOnConnect( (d->statusOnConnect == Yahoo::StatusInvisible) ? Yahoo::StatusInvisible : Yahoo::StatusAvailable );
 	d->loginTask->go();
@@ -173,7 +177,7 @@ void Client::cs_connected()
 
 void Client::close()
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << endl;
+	kDebug(YAHOO_RAW_DEBUG) ;
 	m_pingTimer->stop();
 	if( d->active )
 	{
@@ -211,18 +215,18 @@ QString Client::errorInformation()
 // SLOTS //
 void Client::streamError( int error )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "CLIENT ERROR (Error " <<  error << ")" << endl;
+	kDebug(YAHOO_RAW_DEBUG) << "CLIENT ERROR (Error " <<  error << ")";
 	QString msg;
 
 	d->active = false;
 
 	// Examine error
-	if( error == ClientStream::ErrConnection )			// Ask Connector in this case
+	if( error == ClientStream::ErrConnection && m_connector )			// Ask Connector in this case
 	{
 		d->error = m_connector->errorCode();
 		d->errorString = KSocketBase::errorString( (KSocketBase::SocketError)d->error );
 	}
-	else
+	else if( d->stream )
 	{
 		d->error = error;
 		d->errorString = d->stream->errorText();
@@ -243,7 +247,7 @@ void Client::streamReadyRead()
 
 void Client::lt_loginFinished()
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << endl;
+	kDebug(YAHOO_RAW_DEBUG) ;
 
 	slotLoginResponse( d->loginTask->statusCode(), d->loginTask->statusString() );
 }
@@ -256,7 +260,7 @@ void Client::slotLoginResponse( int response, const QString &msg )
 				d->statusOnConnect == Yahoo::StatusInvisible) ||
 				!d->statusMessageOnConnect.isEmpty() )
 			changeStatus( d->statusOnConnect, d->statusMessageOnConnect, Yahoo::StatusTypeAway );
-		d->statusMessageOnConnect = QString::null;
+		d->statusMessageOnConnect.clear();
 		setStatus( d->statusOnConnect );
 		m_pingTimer->start( 60 * 1000 );
 		initTasks();
@@ -265,19 +269,19 @@ void Client::slotLoginResponse( int response, const QString &msg )
 		close();
 	}
 
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "Emitting loggedIn" << endl;
+	kDebug(YAHOO_RAW_DEBUG) << "Emitting loggedIn";
 	emit loggedIn( response, msg );
 }
 
 void Client::lt_gotSessionID( uint id )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "Got SessionID: " << id << endl;	
+	kDebug(YAHOO_RAW_DEBUG) << "Got SessionID: " << id;	
 	d->sessionID = id;
 }
 
 void Client::slotGotCookies()
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "Y: " << d->loginTask->yCookie()
+	kDebug(YAHOO_RAW_DEBUG) << "Y: " << d->loginTask->yCookie()
 					<< " T: " << d->loginTask->tCookie()
 					<< " C: " << d->loginTask->cCookie() << endl;
 	d->yCookie = d->loginTask->yCookie();
@@ -326,12 +330,12 @@ void Client::sendBuzz( const QString &to )
 {
 	SendMessageTask *smt = new SendMessageTask( d->root );
 	smt->setTarget( to );
-	smt->setText( QString::fromLatin1( "<ding>" ) );
+	smt->setText( QLatin1String( "<ding>" ) );
 	smt->setPicureFlag( pictureFlag() );
 	smt->go( true );
 }
 
-void Client::sendFile( unsigned int transferId, const QString &to, const QString &msg, KURL url )
+void Client::sendFile( unsigned int transferId, const QString &to, const QString &msg, KUrl url )
 {
 	SendFileTask *sft = new SendFileTask( d->root );
 
@@ -348,7 +352,7 @@ void Client::sendFile( unsigned int transferId, const QString &to, const QString
 	sft->go( true );
 }
 
-void Client::receiveFile( unsigned int transferId, const QString &userId, KURL remoteURL, KURL localURL )
+void Client::receiveFile( unsigned int transferId, const QString &userId, KUrl remoteURL, KUrl localURL )
 {
 	ReceiveFileTask *rft = new ReceiveFileTask( d->root );
 
@@ -368,7 +372,7 @@ void Client::receiveFile( unsigned int transferId, const QString &userId, KURL r
 	rft->go( true );
 }
 
-void Client::rejectFile( const QString &userId, KURL remoteURL )
+void Client::rejectFile( const QString &userId, KUrl remoteURL )
 {
 	if( remoteURL.url().startsWith( "http://" ) )
 		return;
@@ -388,7 +392,7 @@ void Client::cancelFileTransfer( unsigned int transferId )
 
 void Client::changeStatus( Yahoo::Status status, const QString &message, Yahoo::StatusType type )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "status: " << status
+	kDebug(YAHOO_RAW_DEBUG) << "status: " << status
 					<< " message: " << message
 					<< " type: " << type << endl;	
 	ChangeStatusTask *cst = new ChangeStatusTask( d->root );
@@ -398,7 +402,7 @@ void Client::changeStatus( Yahoo::Status status, const QString &message, Yahoo::
 	cst->go( true );
 	
 	if( status == Yahoo::StatusInvisible )
-		stealthContact( QString::null, Yahoo::StealthOnline, Yahoo::StealthClear );
+		stealthContact( QString(), Yahoo::StealthOnline, Yahoo::StealthClear );
 
 	setStatus( status );
 }
@@ -416,10 +420,10 @@ void Client::sendPing()
 {
 	if( !d->active )
 	{
-		kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "Disconnected. NOT sending a PING." << endl;
+		kDebug(YAHOO_RAW_DEBUG) << "Disconnected. NOT sending a PING.";
 		return;
 	}
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "Sending a PING." << endl;
+	kDebug(YAHOO_RAW_DEBUG) << "Sending a PING.";
 	PingTask *pt = new PingTask( d->root );
 	pt->go( true );
 }
@@ -473,21 +477,21 @@ void Client::requestPicture( const QString &userId )
 	rpt->go( true );
 }
 
-void Client::downloadPicture(  const QString &userId, KURL url, int checksum )
+void Client::downloadPicture(  const QString &userId, KUrl url, int checksum )
 {
 	if( !d->iconLoader )
 	{
 		d->iconLoader = new YahooBuddyIconLoader( this );
-		QObject::connect( d->iconLoader, SIGNAL(fetchedBuddyIcon(const QString&, KTempFile*, int )),
-				SIGNAL(pictureDownloaded(const QString&, KTempFile*,  int ) ) );
+		QObject::connect( d->iconLoader, SIGNAL(fetchedBuddyIcon(const QString&, KTemporaryFile*, int )),
+				SIGNAL(pictureDownloaded(const QString&, KTemporaryFile*,  int ) ) );
 	}
 
-	d->iconLoader->fetchBuddyIcon( QString(userId), KURL(url), checksum );
+	d->iconLoader->fetchBuddyIcon( QString(userId), KUrl(url), checksum );
 }
 
-void Client::uploadPicture( KURL url )
+void Client::uploadPicture( KUrl url )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "URL: " << url.url() << endl;
+	kDebug(YAHOO_RAW_DEBUG) << "URL: " << url.url();
 	SendPictureTask *spt = new SendPictureTask( d->root );
 	spt->setType( SendPictureTask::UploadPicture );
 	spt->setFilename( url.fileName() );
@@ -495,24 +499,23 @@ void Client::uploadPicture( KURL url )
 		spt->setPath( url.path() );
 	else
 		spt->setPath( url.url() );
-	d->pictureFlag = 2;
 	spt->go( true );
 }
 
-void Client::sendPictureChecksum( int checksum, const QString &who )
+void Client::sendPictureChecksum( const QString &userId, int checksum )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "checksum: " << checksum << endl;
+	kDebug(YAHOO_RAW_DEBUG) << "checksum: " << checksum;
 	SendPictureTask *spt = new SendPictureTask( d->root );
 	spt->setType( SendPictureTask::SendChecksum );
 	spt->setChecksum( checksum );
-	if( !who.isEmpty() )
-		spt->setTarget( who );
+	if( !userId.isEmpty() )
+		spt->setTarget( userId );
 	spt->go( true );	
 }
 
 void Client::sendPictureInformation( const QString &userId, const QString &url, int checksum )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "checksum: " << checksum << endl;
+	kDebug(YAHOO_RAW_DEBUG) << "checksum: " << checksum;
 	SendPictureTask *spt = new SendPictureTask( d->root );
 	spt->setType( SendPictureTask::SendInformation );
 	spt->setChecksum( checksum );
@@ -521,13 +524,16 @@ void Client::sendPictureInformation( const QString &userId, const QString &url,
 	spt->go( true );
 }
 
-void Client::sendPictureStatusUpdate( const QString &userId, int type )
+void Client::setPictureStatus( Yahoo::PictureStatus status )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << "Setting PictureStatus to: " << type << endl;
+	if( d->pictureFlag == status )
+		return;
+
+	kDebug(YAHOO_RAW_DEBUG) << "Setting PictureStatus to: " << status;
+	d->pictureFlag = status;
 	SendPictureTask *spt = new SendPictureTask( d->root );
 	spt->setType( SendPictureTask::SendStatus );
-	spt->setStatus( type );
-	spt->setTarget( userId );
+	spt->setStatus( status );
 	spt->go( true );
 }
 
@@ -624,10 +630,36 @@ void Client::deleteYABEntry(  YABEntry &entry )
 	myt->go(true);
 }
 
+// ***** Yahoo Chat *****
+void Client::getYahooChatCategories()
+{
+	d->yahooChatTask->getYahooChatCategories();
+}
+
+void Client::getYahooChatRooms( const Yahoo::ChatCategory &category )
+{
+	d->yahooChatTask->getYahooChatRooms( category );
+}
+
+void Client::joinYahooChatRoom( const Yahoo::ChatRoom &room )
+{
+	d->yahooChatTask->joinRoom( room );
+}
+
+void Client::sendYahooChatMessage( const QString &msg, const QString &handle )
+{
+	d->yahooChatTask->sendYahooChatMessage( msg, handle );
+}
+
+void Client::leaveChat()
+{
+	d->yahooChatTask->logout();
+}
+
 // ***** other *****
 void Client::notifyError( const QString &info, const QString & errorString, LogLevel level )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << QString::fromLatin1("\nThe following error occured: %1\n    Reason: %2\n    LogLevel: %3")
+	kDebug(YAHOO_RAW_DEBUG) << QString::fromLatin1("\nThe following error occurred: %1\n    Reason: %2\n    LogLevel: %3")
 		.arg(info).arg(errorString).arg(level) << endl;
 	d->errorString = errorString;
 	d->errorInformation = info;
@@ -675,12 +707,6 @@ QString Client::password()
 	return d->pass;
 }
 
-QCString Client::ipAddress()
-{
-	//TODO determine ip address
-	return "127.0.0.1";
-}
-
 QString Client::host()
 {
 	return d->host;
@@ -701,9 +727,14 @@ int Client::pictureFlag()
 	return d->pictureFlag;
 }
 
-void Client::setPictureFlag( int flag )
+int Client::pictureChecksum()
 {
-	d->pictureFlag = flag;
+	return d->pictureChecksum;
+}
+
+void Client::setPictureChecksum( int cs )
+{
+	d->pictureChecksum = cs;
 }
 
 QString Client::yCookie()
@@ -723,18 +754,18 @@ QString Client::cCookie()
 
 void Client::distribute( Transfer * transfer )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << k_funcinfo << endl;
+	kDebug(YAHOO_RAW_DEBUG) ;
 	if( !rootTask()->take( transfer ) )
-		kdDebug(YAHOO_RAW_DEBUG) << "CLIENT: root task refused transfer" << endl;
+		kDebug(YAHOO_RAW_DEBUG) << "CLIENT: root task refused transfer";
 	delete transfer;
 }
 
 void Client::send( Transfer* request )
 {
-	kdDebug(YAHOO_RAW_DEBUG) << "CLIENT::send()"<< endl;
+	kDebug(YAHOO_RAW_DEBUG) << "CLIENT::send()";
 	if( !d->stream )
 	{	
-		kdDebug(YAHOO_RAW_DEBUG) << "CLIENT - NO STREAM TO SEND ON!" << endl;
+		kDebug(YAHOO_RAW_DEBUG) << "CLIENT - NO STREAM TO SEND ON!";
 		return;
 	}
 
@@ -743,7 +774,7 @@ void Client::send( Transfer* request )
 
 void Client::debug(const QString &str)
 {
-	qDebug( "CLIENT: %s", str.ascii() );
+       qDebug( "CLIENT: %s", qPrintable(str) );
 }
 
 Task * Client::rootTask()
@@ -757,8 +788,8 @@ void Client::initTasks()
 		return;
 
 	d->statusTask = new StatusNotifierTask( d->root );
-	QObject::connect( d->statusTask, SIGNAL( statusChanged( const QString&, int, const QString&, int, int ) ), 
-				SIGNAL( statusChanged( const QString&, int, const QString&, int, int ) ) );
+	QObject::connect( d->statusTask, SIGNAL( statusChanged(QString,int,const QString,int,int,int) ), 
+				SIGNAL( statusChanged(QString,int,const QString,int,int,int) ) );
 	QObject::connect( d->statusTask, SIGNAL( stealthStatusChanged( const QString&, Yahoo::StealthStatus ) ), 
 				SIGNAL( stealthStatusChanged( const QString&, Yahoo::StealthStatus ) ) );
 	QObject::connect( d->statusTask, SIGNAL( loginResponse( int, const QString& ) ), 
@@ -769,8 +800,6 @@ void Client::initTasks()
 				SIGNAL( authorizationAccepted( const QString& ) ) );
 	QObject::connect( d->statusTask, SIGNAL( gotAuthorizationRequest( const QString &, const QString &, const QString & ) ), 
 				SIGNAL( gotAuthorizationRequest( const QString &, const QString &, const QString & ) ) );
-	QObject::connect( d->statusTask, SIGNAL( gotPictureChecksum( const QString &, int ) ),
-				SIGNAL( pictureChecksumNotify( const QString &, int ) ) );
 
 	d->mailTask = new MailNotifierTask( d->root );
 	QObject::connect( d->mailTask, SIGNAL( mailNotify(const QString&, const QString&, int) ), 
@@ -793,12 +822,12 @@ void Client::initTasks()
 				SIGNAL( pictureStatusNotify( const QString &, int ) ) );
 	QObject::connect( d->pictureNotifierTask, SIGNAL( pictureChecksumNotify( const QString &, int ) ),
 				SIGNAL( pictureChecksumNotify( const QString &, int ) ) );
-	QObject::connect( d->pictureNotifierTask, SIGNAL( pictureInfoNotify( const QString &, KURL, int ) ),
-				SIGNAL( pictureInfoNotify( const QString &, KURL, int ) ) );
+	QObject::connect( d->pictureNotifierTask, SIGNAL( pictureInfoNotify( const QString &, KUrl, int ) ),
+				SIGNAL( pictureInfoNotify( const QString &, KUrl, int ) ) );
 	QObject::connect( d->pictureNotifierTask, SIGNAL( pictureRequest( const QString & ) ),
 				SIGNAL( pictureRequest( const QString & ) ) );
-	QObject::connect( d->pictureNotifierTask, SIGNAL( pictureUploaded( const QString & ) ),
-				SIGNAL( pictureUploaded( const QString & ) ) );
+	QObject::connect( d->pictureNotifierTask, SIGNAL( pictureUploaded( const QString &, int ) ),
+				SIGNAL( pictureUploaded( const QString &, int ) ) );
 
 	d->webcamTask = new WebcamTask( d->root );
 	QObject::connect( d->webcamTask, SIGNAL( webcamImageReceived( const QString &, const QPixmap &) ),
@@ -840,9 +869,23 @@ void Client::initTasks()
 
 	d->fileTransferTask = new FileTransferNotifierTask( d->root );
 	QObject::connect( d->fileTransferTask, SIGNAL(incomingFileTransfer( const QString &, const QString &, 
-					long, const QString &, const QString &, unsigned long )),
+					long, const QString &, const QString &, unsigned long, const QPixmap & )),
 				SIGNAL(incomingFileTransfer( const QString &, const QString &, 
-					long, const QString &, const QString &, unsigned long )) );
+					long, const QString &, const QString &, unsigned long, const QPixmap & )) );
+
+	d->yahooChatTask = new YahooChatTask( d->root );
+	QObject::connect( d->yahooChatTask, SIGNAL(gotYahooChatCategories( const QDomDocument & )),
+				SIGNAL(gotYahooChatCategories( const QDomDocument & )) );
+	QObject::connect( d->yahooChatTask, SIGNAL(gotYahooChatRooms( const Yahoo::ChatCategory &, const QDomDocument & )),
+				SIGNAL(gotYahooChatRooms( const Yahoo::ChatCategory &, const QDomDocument & )) );
+	QObject::connect( d->yahooChatTask, SIGNAL(chatRoomJoined( int , int , const QString &, const QString & ) ),
+				SIGNAL(chatRoomJoined( int , int , const QString &, const QString & ) ) );
+	QObject::connect( d->yahooChatTask, SIGNAL(chatBuddyHasJoined( const QString &, const QString &, bool  ) ),
+				SIGNAL(chatBuddyHasJoined( const QString &, const QString &, bool  ) ) );
+	QObject::connect( d->yahooChatTask, SIGNAL(chatBuddyHasLeft(QString,QString) ),
+				SIGNAL(chatBuddyHasLeft(QString,QString) ) );
+	QObject::connect( d->yahooChatTask, SIGNAL(chatMessageReceived( const QString &, const QString &, const QString & ) ),
+				SIGNAL(chatMessageReceived( const QString &, const QString &, const QString & ) ) );
 }
 
 void Client::deleteTasks()
@@ -864,6 +907,8 @@ void Client::deleteTasks()
 	d->yabTask = 0L;
 	d->fileTransferTask->deleteLater();
 	d->fileTransferTask = 0;
+	d->yahooChatTask->deleteLater();
+	d->yahooChatTask = 0;
 }
 
 #include "client.moc"
