@@ -955,7 +955,7 @@ QString YahooAccount::prepareIncomingMessage( const QString &messageText )
 		pos = regExp.indexIn( newMsgText, pos );
 		if ( pos >= 0 ) {
 			pos += regExp.matchedLength();
-			newMsgText.replace( regExp, QLatin1String("" ) );
+			newMsgText.remove( regExp );
 
 		}
 	}
@@ -965,7 +965,7 @@ QString YahooAccount::prepareIncomingMessage( const QString &messageText )
 		pos = regExp.indexIn( newMsgText, pos );
 		if ( pos >= 0 ) {
 			pos += regExp.matchedLength();
-			newMsgText.replace( regExp, QLatin1String("" ) );
+			newMsgText.remove( regExp );
 		}
 	}
 
@@ -1098,7 +1098,7 @@ void YahooAccount::slotGotConfInvite( const QString & who, const QString & room,
 	QString m = who;
 	QStringList myMembers;
 	myMembers.push_back( who );
-	for( QStringList::const_iterator it = ++members.begin(); it != members.end(); it++ )
+	for( QStringList::const_iterator it = ++members.constBegin(); it != members.constEnd(); ++it )
 	{
 		if( *it != m_session->userId() )
 		{
@@ -1108,7 +1108,7 @@ void YahooAccount::slotGotConfInvite( const QString & who, const QString & room,
 	}
 	if( KMessageBox::Yes == KMessageBox::questionYesNo( Kopete::UI::Global::mainWidget(),
 		i18n("%1 has invited you to join a conference with %2.\n\nHis/her message: %3\n\nAccept?",
-				who, m, msg), QString(), KGuiItem( i18n("Accept") ), KGuiItem( i18n("Ignore") ) ) )
+				who, m, msg), QString(), KGuiItem( i18nc("@action","Accept") ), KGuiItem( i18nc("@action","Ignore") ) ) )
 	{
 		m_session->joinConference( room, myMembers );
 		if( !m_conferences[room] )
@@ -1119,7 +1119,7 @@ void YahooAccount::slotGotConfInvite( const QString & who, const QString & room,
 
 			QObject::connect( session, SIGNAL(leavingConference( YahooConferenceChatSession * ) ), this, SLOT( slotConfLeave( YahooConferenceChatSession * ) ) );
 
-			for ( QStringList::ConstIterator it = myMembers.begin(); it != myMembers.end(); ++it )
+			for ( QStringList::ConstIterator it = myMembers.constBegin(); it != myMembers.constEnd(); ++it )
 			{
 				YahooContact * c = contact( *it );
 				if ( !c )
@@ -1560,7 +1560,7 @@ void YahooAccount::slotGotWebcamInvite( const QString& who )
 	m_pendingWebcamInvites.append( who );
 
 	if( KMessageBox::Yes == KMessageBox::questionYesNo( Kopete::UI::Global::mainWidget(), i18n("%1 has invited you to view his/her webcam. Accept?", who),
-                            QString(), KGuiItem( i18n("Accept") ), KGuiItem( i18n("Ignore") ) ) )
+                            QString(), KGuiItem( i18nc("@action","Accept") ), KGuiItem( i18nc("@action","Ignore") ) ) )
 	{
 		m_pendingWebcamInvites.removeAll( who );
 		m_session->requestWebcam( who );
@@ -1771,7 +1771,7 @@ void YahooAccount::slotWebcamViewerJoined( const QString &viewer )
 void YahooAccount::slotWebcamViewerRequest( const QString &viewer )
 {
 	if( KMessageBox::Yes == KMessageBox::questionYesNo( Kopete::UI::Global::mainWidget(), i18n("%1 wants to view your webcam. Grant access?",
-		viewer), QString(), KGuiItem( i18n("Accept") ), KGuiItem( i18n("Ignore") ) ) )
+		viewer), QString(), KGuiItem( i18nc("@action","Accept") ), KGuiItem( i18nc("@action","Ignore") ) ) )
 		m_session->grantWebcamAccess( viewer );
 }
 
