@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2007 Urs Wolfer <uwolfer @ kde.org>
+** Copyright (C) 2007 - 2008 Urs Wolfer <uwolfer @ kde.org>
 **
 ** This file is part of KDE.
 **
@@ -30,6 +30,7 @@
 #include "bookmarkmanager.h"
 #include "remotedesktopsmodel.h"
 #include "systemtrayicon.h"
+
 #ifdef BUILD_RDP
 #include "rdpview.h"
 #endif
@@ -42,6 +43,9 @@
 #ifdef BUILD_ZEROCONF
 #include "zeroconfpage.h"
 #endif
+#ifdef BUILD_TEST
+#include "testview.h"
+#endif
 
 #include <KAction>
 #include <KActionCollection>
@@ -54,6 +58,7 @@
 #include <KMenuBar>
 #include <KMessageBox>
 #include <KNotifyConfigWidget>
+#include <KProcess>
 #include <KPushButton>
 #include <KShortcut>
 #include <KShortcutsDialog>
@@ -62,6 +67,7 @@
 #include <KToggleAction>
 #include <KToggleFullScreenAction>
 #include <KUrlNavigator>
+#include <KStandardDirs>
 
 #include <QClipboard>
 #include <QCloseEvent>
@@ -177,7 +183,7 @@ void MainWindow::setupActions()
     zeroconfAction->setIcon(KIcon("network-connect"));
     connect(zeroconfAction, SIGNAL(triggered()), SLOT(createZeroconfPage()));
 #ifndef BUILD_ZEROCONF
-    zeroconfAction->setVisible(false);
+    zeroconfAction->deleteLater();
 #endif
 
     QAction *screenshotAction = actionCollection()->addAction("take_screenshot");
@@ -196,10 +202,10 @@ void MainWindow::setupActions()
     viewOnlyAction->setIcon(KIcon("document-preview"));
     connect(viewOnlyAction, SIGNAL(triggered(bool)), SLOT(viewOnly(bool)));
 
-    QAction *logoutAction = actionCollection()->addAction("logout");
-    logoutAction->setText(i18n("Log Out"));
-    logoutAction->setIcon(KIcon("system-log-out"));
-    connect(logoutAction, SIGNAL(triggered()), SLOT(logout()));
+    QAction *disconnectAction = actionCollection()->addAction("disconnect");
+    disconnectAction->setText(i18n("Disconnect"));
+    disconnectAction->setIcon(KIcon("system-log-out"));
+    connect(disconnectAction, SIGNAL(triggered()), SLOT(disconnect()));
 
     QAction *showLocalCursorAction = actionCollection()->addAction("show_local_cursor");
     showLocalCursorAction->setCheckable(true);
@@ -219,20 +225,14 @@ void MainWindow::setupActions()
     scaleAction->setText(i18n("Scale remote screen to fit window size"));
     connect(scaleAction, SIGNAL(triggered(bool)), SLOT(scale(bool)));
 
-    QAction *quitAction = KStandardAction::quit(this, SLOT(quit()), actionCollection());
-    actionCollection()->addAction("quit", quitAction);
-    QAction *preferencesAction = KStandardAction::preferences(this, SLOT(preferences()), actionCollection());
-    actionCollection()->addAction("preferences", preferencesAction);
-    QAction *configToolbarAction = KStandardAction::configureToolbars(this, SLOT(configureToolbars()), actionCollection());
-    actionCollection()->addAction("configure_toolbars", configToolbarAction);
-    QAction *keyBindingsAction = KStandardAction::keyBindings(this, SLOT(configureKeys()), actionCollection());
-    actionCollection()->addAction("configure_keys", keyBindingsAction);
-    QAction *cinfigNotifyAction = KStandardAction::configureNotifications(this, SLOT(configureNotifications()), actionCollection());
-    cinfigNotifyAction->setVisible(false);
-    actionCollection()->addAction("configure_notifications", cinfigNotifyAction);
+    KStandardAction::quit(this, SLOT(quit()), actionCollection());
+    KStandardAction::preferences(this, SLOT(preferences()), actionCollection());
+    KStandardAction::configureToolbars(this, SLOT(configureToolbars()), actionCollection());
+    KStandardAction::keyBindings(this, SLOT(configureKeys()), actionCollection());
+    QAction *configNotifyAction = KStandardAction::configureNotifications(this, SLOT(configureNotifications()), actionCollection());
+    configNotifyAction->setVisible(false);
     m_menubarAction = KStandardAction::showMenubar(this, SLOT(showMenubar()), actionCollection());
     m_menubarAction->setChecked(!menuBar()->isHidden());
-    actionCollection()->addAction("settings_showmenubar", m_menubarAction);
 
     QString initialProtocol;
 #ifdef BUILD_RDP
@@ -255,6 +255,9 @@ void MainWindow::setupActions()
 #endif
 #ifdef BUILD_RDP
                                            << "rdp"
+#endif
+#ifdef BUILD_TEST
+                                           << "test"
 #endif
                                           );
     m_addressNavigator->setUrlEditable(Settings::normalUrlInputLine());
@@ -285,9 +288,9 @@ void MainWindow::setupActions()
 
 void MainWindow::restoreOpenSessions()
 {
-    QStringList list = Settings::openSessions();
-    QStringList::Iterator it = list.begin();
-    QStringList::Iterator end = list.end();
+    const QStringList list = Settings::openSessions();
+    QStringList::ConstIterator it = list.begin();
+    QStringList::ConstIterator end = list.end();
     while (it != end) {
         newConnection(*it);
         ++it;
@@ -298,7 +301,7 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
 {
     m_switchFullscreenWhenConnected = switchFullscreenWhenConnected;
 
-    KUrl url = newUrl.isEmpty() ? m_addressNavigator->uncommittedUrl() : newUrl;
+    const KUrl url = newUrl.isEmpty() ? m_addressNavigator->uncommittedUrl() : newUrl;
 
     if (!url.isValid() || (url.host().isEmpty() && url.port() < 0)
             || !url.path().isEmpty()) {
@@ -311,18 +314,23 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
     m_addressNavigator->setUrl(KUrl(url.scheme().toLower() + "://"));
 
     RemoteView *view = 0;
+    KConfigGroup configGroup = Settings::self()->config()->group("hostpreferences").group(url.prettyUrl(KUrl::RemoveTrailingSlash));
 
-    if (url.scheme().toLower() == "vnc") {
+    if (url.scheme().compare("vnc", Qt::CaseInsensitive) == 0) {
 #ifdef BUILD_VNC
-        view = new VncView(this, url);
+        view = new VncView(this, url, configGroup);
 #endif
-    } else if (url.scheme().toLower() == "nx") {
+    } else if (url.scheme().compare("nx", Qt::CaseInsensitive) == 0) {
 #ifdef BUILD_NX
-        view = new NxView(this, url);
+        view = new NxView(this, url, configGroup);
 #endif
-    } else if (url.scheme().toLower() == "rdp") {
+    } else if (url.scheme().compare("rdp", Qt::CaseInsensitive) == 0) {
 #ifdef BUILD_RDP
-        view = new RdpView(this, url);
+        view = new RdpView(this, url, configGroup);
+#endif
+    } else if (url.scheme().compare("test", Qt::CaseInsensitive) == 0) {
+#ifdef BUILD_TEST
+        view = new TestView(this, url, configGroup);
 #endif
     } else
     {
@@ -338,23 +346,42 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
                 i18n("Unusable URL"));
         return;
     }
+    
+    // Configure the view
+    HostPreferences* prefs = view->hostPreferences();
+    if (! prefs->showDialogIfNeeded()) {
+        delete view;
+        return;
+    }
+    
+    saveHostPrefs();
+    
+    view->setGrabAllKeys(prefs->grabAllKeys());
+    view->showDotCursor(prefs->showLocalCursor() ? RemoteView::CursorOn : RemoteView::CursorOff);
+    view->setViewOnly(prefs->viewOnly());
+    if (! switchFullscreenWhenConnected) view->enableScaling(prefs->windowedScale()); 
 
-    connect(view, SIGNAL(changeSize(int, int)), this, SLOT(resizeTabWidget(int, int)));
+    connect(view, SIGNAL(framebufferSizeChanged(int, int)), this, SLOT(resizeTabWidget(int, int)));
     connect(view, SIGNAL(statusChanged(RemoteView::RemoteStatus)), this, SLOT(statusChanged(RemoteView::RemoteStatus)));
 
     m_remoteViewList.append(view);
 
-    view->resize(0, 0);
+//     view->resize(0, 0);
  
     int numNonRemoteView = 0;
     if (m_showStartPage)
         numNonRemoteView++;
     if (m_zeroconfPage)
         numNonRemoteView++;
+    
+    if (m_remoteViewList.size() == 1) {
+        kDebug(5010) << "First connection, restoring window size)";
+        restoreWindowSize(view->hostPreferences()->configGroup());
+    }
 
-    QScrollArea *scrollArea = createScrollArea(m_tabWidget, m_remoteViewList.at(m_remoteViewList.count() - 1));
+    QScrollArea *scrollArea = createScrollArea(m_tabWidget, view);
 
-    int newIndex = m_tabWidget->addTab(scrollArea, KIcon("krdc"), url.prettyUrl(KUrl::RemoveTrailingSlash));
+    const int newIndex = m_tabWidget->addTab(scrollArea, KIcon("krdc"), url.prettyUrl(KUrl::RemoveTrailingSlash));
     m_tabWidget->setCurrentIndex(newIndex);
     tabChanged(newIndex); // force to update m_currentRemoteView (tabChanged is not emitted when start page has been disabled)
 
@@ -363,11 +390,11 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
 
 void MainWindow::openFromDockWidget(const QModelIndex &index)
 {
-    QString data = index.data(Qt::UserRole).toString();
+    const QString data = index.data(Qt::UserRole).toString();
     if (!data.isEmpty()) {
-        KUrl url(data);
+        const KUrl url(data);
         // first check if url has already been opened; in case show the tab
-        for (int i = 0; i < m_remoteViewList.count(); i++) {
+        for (int i = 0; i < m_remoteViewList.count(); ++i) {
             if (m_remoteViewList.at(i)->url() == url) {
                 int numNonRemoteView = 0;
                 if (m_showStartPage)
@@ -385,6 +412,9 @@ void MainWindow::openFromDockWidget(const QModelIndex &index)
 void MainWindow::resizeTabWidget(int w, int h)
 {
     kDebug(5010) << "tabwidget resize: w: " << w << ", h: " << h;
+    
+    RemoteView* view = m_currentRemoteView >= 0 ? m_remoteViewList.at(m_currentRemoteView) : NULL;
+    if (view && view->scaling()) return;
 
     if (m_topBottomBorder == 0) { // the values are not cached yet
         QScrollArea *tmp = qobject_cast<QScrollArea *>(m_tabWidget->currentWidget());
@@ -395,12 +425,12 @@ void MainWindow::resizeTabWidget(int w, int h)
         kDebug(5010) << "tabwidget border: w: " << m_leftRightBorder << ", h: " << m_topBottomBorder;
     }
 
-    int newTabWidth = w + m_leftRightBorder;
-    int newTabHeight = h + m_topBottomBorder;
+    const int newTabWidth = w + m_leftRightBorder;
+    const int newTabHeight = h + m_topBottomBorder;
 
-    QSize newWindowSize = size() - m_tabWidget->size() + QSize(newTabWidth, newTabHeight);
+    const QSize newWindowSize = size() - m_tabWidget->size() + QSize(newTabWidth, newTabHeight);
 
-    QSize desktopSize = QSize(QApplication::desktop()->availableGeometry().width(),
+    const QSize desktopSize = QSize(QApplication::desktop()->availableGeometry().width(),
                               QApplication::desktop()->availableGeometry().height());
 
     if ((newWindowSize.height() >= desktopSize.height()) || (newWindowSize.width() >= desktopSize.width())) {
@@ -424,7 +454,7 @@ void MainWindow::statusChanged(RemoteView::RemoteStatus status)
     if (status == RemoteView::Disconnecting || status == RemoteView::Disconnected)
         return;
 
-    QString host = m_remoteViewList.at(m_currentRemoteView)->host();
+    const QString host = m_remoteViewList.at(m_currentRemoteView)->host();
 
     QString iconName = "krdc";
     QString message;
@@ -465,7 +495,7 @@ void MainWindow::statusChanged(RemoteView::RemoteStatus status)
 
 void MainWindow::takeScreenshot()
 {
-    QPixmap snapshot = QPixmap::grabWidget(m_remoteViewList.at(m_currentRemoteView));
+    const QPixmap snapshot = QPixmap::grabWidget(m_remoteViewList.at(m_currentRemoteView));
 
     QApplication::clipboard()->setPixmap(snapshot);
 }
@@ -474,22 +504,25 @@ void MainWindow::switchFullscreen()
 {
     kDebug(5010);
 
+    RemoteView* view = m_remoteViewList.at(m_currentRemoteView);
+
     if (m_fullscreenWindow) {
+        // Leaving fullscreen mode
+        view->enableScaling(view->hostPreferences()->windowedScale());
         show();
         restoreGeometry(m_mainWindowGeometry);
 
         m_fullscreenWindow->setWindowState(0);
         m_fullscreenWindow->hide();
 
-        QScrollArea *scrollArea = createScrollArea(m_tabWidget, m_remoteViewList.at(m_currentRemoteView));
+        QScrollArea *scrollArea = createScrollArea(m_tabWidget, view);
 
-        int currentTab = m_tabWidget->currentIndex();
+        const int currentTab = m_tabWidget->currentIndex();
         m_tabWidget->insertTab(currentTab, scrollArea, m_tabWidget->tabIcon(currentTab), m_tabWidget->tabText(currentTab));
         m_tabWidget->removeTab(m_tabWidget->currentIndex());
         m_tabWidget->setCurrentIndex(currentTab);
 
-        resizeTabWidget(m_remoteViewList.at(m_currentRemoteView)->sizeHint().width(),
-                        m_remoteViewList.at(m_currentRemoteView)->sizeHint().height());
+        resizeTabWidget(view->sizeHint().width(), view->sizeHint().height());
 
         if (m_toolBar) {
             m_toolBar->hideAndDestroy();
@@ -504,11 +537,12 @@ void MainWindow::switchFullscreen()
 
         m_fullscreenWindow = 0;
     } else {
+        // Entering fullscreen mode
         m_fullscreenWindow = new QWidget(this, Qt::Window);
         m_fullscreenWindow->setWindowTitle(i18nc("window title when in fullscreen mode (for example displayed in tasklist)",
                                                  "KDE Remote Desktop Client (Fullscreen)"));
 
-        QScrollArea *scrollArea = createScrollArea(m_fullscreenWindow, m_remoteViewList.at(m_currentRemoteView));
+        QScrollArea *scrollArea = createScrollArea(m_fullscreenWindow, view);
         scrollArea->setFrameShape(QFrame::NoFrame);
 
         QVBoxLayout *fullscreenLayout = new QVBoxLayout(m_fullscreenWindow);
@@ -519,6 +553,8 @@ void MainWindow::switchFullscreen()
         minimizePixel->winId(); // force it to be a native widget (prevents problem with QX11EmbedContainer)
         connect(minimizePixel, SIGNAL(rightClicked()), m_fullscreenWindow, SLOT(showMinimized()));
 
+        view->enableScaling(view->hostPreferences()->fullscreenScale());
+                
         m_fullscreenWindow->show();
 
         KToggleFullScreenAction::setFullScreen(m_fullscreenWindow, true);
@@ -547,7 +583,7 @@ QScrollArea *MainWindow::createScrollArea(QWidget *parent, RemoteView *remoteVie
     return scrollArea;
 }
 
-void MainWindow::logout()
+void MainWindow::disconnect()
 {
     kDebug(5010);
 
@@ -556,17 +592,16 @@ void MainWindow::logout()
     }
 
     QWidget *tmp = m_tabWidget->currentWidget();
-
     m_remoteViewList.removeAt(m_currentRemoteView);
-
     m_tabWidget->removeTab(m_tabWidget->currentIndex());
-
     tmp->deleteLater();
+    
+    saveHostPrefs();
 }
 
 void MainWindow::closeTab(QWidget *widget)
 {
-    int index = m_tabWidget->indexOf(widget);
+    const int index = m_tabWidget->indexOf(widget);
 
     kDebug(5010) << index;
 
@@ -601,28 +636,44 @@ void MainWindow::showLocalCursor(bool showLocalCursor)
 {
     kDebug(5010) << showLocalCursor;
 
-    m_remoteViewList.at(m_currentRemoteView)->showDotCursor(showLocalCursor ? RemoteView::CursorOn : RemoteView::CursorOff);
+    RemoteView* view = m_remoteViewList.at(m_currentRemoteView);
+    view->showDotCursor(showLocalCursor ? RemoteView::CursorOn : RemoteView::CursorOff);
+    view->hostPreferences()->setViewOnly(showLocalCursor);
+    saveHostPrefs();
 }
 
 void MainWindow::viewOnly(bool viewOnly)
 {
     kDebug(5010) << viewOnly;
 
-    m_remoteViewList.at(m_currentRemoteView)->setViewOnly(viewOnly);
+    RemoteView* view = m_remoteViewList.at(m_currentRemoteView);
+    view->setViewOnly(viewOnly);
+    view->hostPreferences()->setViewOnly(viewOnly);
+    saveHostPrefs();
 }
 
 void MainWindow::grabAllKeys(bool grabAllKeys)
 {
     kDebug(5010);
 
-    m_remoteViewList.at(m_currentRemoteView)->setGrabAllKeys(grabAllKeys);
+    RemoteView* view = m_remoteViewList.at(m_currentRemoteView);
+    view->setGrabAllKeys(grabAllKeys);
+    view->hostPreferences()->setGrabAllKeys(grabAllKeys);
+    saveHostPrefs();
 }
 
 void MainWindow::scale(bool scale)
 {
     kDebug(5010);
 
-    m_remoteViewList.at(m_currentRemoteView)->enableScaling(scale);
+    RemoteView* view = m_remoteViewList.at(m_currentRemoteView);
+    view->enableScaling(scale);
+    if (m_fullscreenWindow) 
+        view->hostPreferences()->setFullscreenScale(scale);
+    else
+        view->hostPreferences()->setWindowedScale(scale);
+    
+    saveHostPrefs();
 }
 
 void MainWindow::showRemoteViewToolbar()
@@ -668,7 +719,7 @@ void MainWindow::showRemoteViewToolbar()
         m_toolBar->addAction(actionCollection()->action("show_local_cursor"));
         m_toolBar->addAction(actionCollection()->action("grab_all_keys"));
         m_toolBar->addAction(actionCollection()->action("scale"));
-        m_toolBar->addAction(actionCollection()->action("logout"));
+        m_toolBar->addAction(actionCollection()->action("disconnect"));
 
         QAction *stickToolBarAction = new QAction(m_toolBar);
         stickToolBarAction->setCheckable(true);
@@ -681,6 +732,13 @@ void MainWindow::showRemoteViewToolbar()
     m_toolBar->showAndAnimate();
 }
 
+void setActionStatus(QAction* action, bool enabled, bool visible, bool checked) 
+{
+    action->setEnabled(enabled);
+    action->setVisible(visible);
+    action->setChecked(checked);
+}
+
 void MainWindow::updateActionStatus()
 {
     kDebug(5010) << m_tabWidget->currentIndex();
@@ -696,32 +754,31 @@ void MainWindow::updateActionStatus()
     else
         enabled = true;
 
+    RemoteView* view = m_currentRemoteView >= 0 ? m_remoteViewList.at(m_currentRemoteView) : NULL;
+    
     actionCollection()->action("switch_fullscreen")->setEnabled(enabled);
     actionCollection()->action("take_screenshot")->setEnabled(enabled);
-    actionCollection()->action("view_only")->setEnabled(enabled);
-    actionCollection()->action("grab_all_keys")->setEnabled(enabled);
-    actionCollection()->action("scale")->setEnabled(enabled);
-    actionCollection()->action("logout")->setEnabled(enabled);
+    actionCollection()->action("disconnect")->setEnabled(enabled);
 
-    bool viewOnlyChecked = false;
-    if (m_currentRemoteView >= 0)
-        viewOnlyChecked = enabled && m_remoteViewList.at(m_currentRemoteView)->viewOnly();
-    actionCollection()->action("view_only")->setChecked(viewOnlyChecked);
+    setActionStatus(actionCollection()->action("view_only"),
+                    enabled, 
+                    true,
+                    view ? view->viewOnly() : false);
+                       
+    setActionStatus(actionCollection()->action("show_local_cursor"),
+                    enabled, 
+                    view ? view->supportsLocalCursor() : false,
+                    view ? view->dotCursorState() == RemoteView::CursorOn : false);
 
-    bool showLocalCursorChecked = false;
-    if (m_currentRemoteView >= 0)
-        showLocalCursorChecked = enabled && m_remoteViewList.at(m_currentRemoteView)->dotCursorState() == RemoteView::CursorOn;
-    actionCollection()->action("show_local_cursor")->setChecked(showLocalCursorChecked);
-
-    bool showLocalCursorVisible = false;
-    if (m_currentRemoteView >= 0)
-        showLocalCursorVisible = enabled && m_remoteViewList.at(m_currentRemoteView)->supportsLocalCursor();
-    actionCollection()->action("show_local_cursor")->setVisible(showLocalCursorVisible);
-
-    bool scaleVisible = false;
-    if (m_currentRemoteView >= 0)
-        scaleVisible = enabled && m_remoteViewList.at(m_currentRemoteView)->supportsScaling();
-    actionCollection()->action("scale")->setVisible(scaleVisible);
+    setActionStatus(actionCollection()->action("scale"),
+                    enabled,
+                    view ? view->supportsScaling() : false,
+                    view ? view->scaling() : false);
+                       
+    setActionStatus(actionCollection()->action("grab_all_keys"),
+                    enabled,
+                    enabled,
+                    view ? view->grabAllKeys() : false);
 }
 
 void MainWindow::preferences()
@@ -770,16 +827,22 @@ void MainWindow::updateConfiguration()
         numNonRemoteView++;
     if (m_zeroconfPage)
         numNonRemoteView++;
-    for (int i = numNonRemoteView; i < m_tabWidget->count(); i++) {
+    for (int i = numNonRemoteView; i < m_tabWidget->count(); ++i) {
         QPalette palette = m_tabWidget->widget(i)->palette();
         palette.setColor(QPalette::Dark, Settings::backgroundColor());
         m_tabWidget->widget(i)->setPalette(palette);
     }
+    
+    // Send update configuration message to all views
+    for (int i = 0; i < m_remoteViewList.count(); ++i) {
+        m_remoteViewList.at(i)->updateConfiguration();
+    }
+
 }
 
 void MainWindow::quit(bool systemEvent)
 {
-    bool haveRemoteConnections = m_remoteViewList.count();
+    const bool haveRemoteConnections = !m_remoteViewList.isEmpty();
     if (systemEvent || !haveRemoteConnections || KMessageBox::warningContinueCancel(this,
                                            i18n("Are you sure you want to quit the KDE Remote Desktop Client?"),
                                            i18n("Confirm Quit"),
@@ -788,13 +851,15 @@ void MainWindow::quit(bool systemEvent)
 
         if (Settings::rememberSessions()) { // remember open remote views for next startup
             QStringList list;
-            for (int i = 0; i < m_remoteViewList.count(); i++) {
+            for (int i = 0; i < m_remoteViewList.count(); ++i) {
                 kDebug(5010) << m_remoteViewList.at(i)->url();
                 list.append(m_remoteViewList.at(i)->url().prettyUrl(KUrl::RemoveTrailingSlash));
             }
             Settings::setOpenSessions(list);
         }
 
+        saveHostPrefs();
+
         Settings::self()->writeConfig();
 
         qApp->quit();
@@ -840,6 +905,23 @@ void MainWindow::closeEvent(QCloseEvent *event)
     }
 }
 
+void MainWindow::saveProperties(KConfigGroup &group)
+{
+    kDebug(5010);
+    KMainWindow::saveProperties(group);
+    saveHostPrefs();
+}
+
+void MainWindow::saveHostPrefs()
+{
+    // Save default window size for currently active view
+    RemoteView* view = m_currentRemoteView >= 0 ? m_remoteViewList.at(m_currentRemoteView) : NULL;
+    if (view)
+        saveWindowSize(view->hostPreferences()->configGroup());
+    
+    Settings::self()->config()->sync();
+}
+
 void MainWindow::tabChanged(int index)
 {
     kDebug(5010) << index;
@@ -852,8 +934,7 @@ void MainWindow::tabChanged(int index)
 
     m_currentRemoteView = index - numNonRemoteView;
 
-    QString tabTitle = m_tabWidget->tabText(index).remove('&');
-
+    const QString tabTitle = m_tabWidget->tabText(index).remove('&');
     setCaption(tabTitle == i18n("Start Page") ? QString() : tabTitle);
 
     updateActionStatus();
@@ -897,11 +978,12 @@ void MainWindow::createStartPage()
     nxConnectButton->setVisible(false);
 #endif
 
-    KPushButton *rdpConnectButton = new KPushButton(this);
+    rdpConnectButton = new KPushButton(this);
     rdpConnectButton->setStyleSheet("KPushButton { padding: 12px; margin: 10px; }");
     rdpConnectButton->setIcon(KIcon(actionCollection()->action("new_rdp_connection")->icon()));
     rdpConnectButton->setText(i18n("Connect to a Windows Remote Desktop (RDP)"));
     connect(rdpConnectButton, SIGNAL(clicked()), SLOT(newRdpConnection()));
+    QMetaObject::invokeMethod(this, "checkRdektopAvailability");
 #ifndef BUILD_RDP
     rdpConnectButton->setVisible(false);
 #endif
@@ -979,10 +1061,18 @@ void MainWindow::closeZeroconfPage()
     m_tabWidget->removeTab(index);
     m_zeroconfPage->deleteLater();
     m_zeroconfPage = 0;
-    tabChanged(index); // force update again because m_zeroconfPage was not null before
 #endif
 }
 
+void MainWindow::checkRdektopAvailability()
+{
+    if (KProcess::execute("rdesktop") < 0) { //-2 if the process could not be started, -1 if it crashed, otherwise its exit code 
+        rdpConnectButton->setEnabled(false);
+        rdpConnectButton->setText(rdpConnectButton->text() + '\n' + 
+                                  i18n("\"rdesktop\" cannot be found on your system; make sure it is properly installed."));
+    }
+}
+
 QList<RemoteView *> MainWindow::remoteViewList() const
 {
     return m_remoteViewList;
